#pragma region File Description
//-----------------------------------------------------------------------------------------------------
// File : AsuraMath.h
//
// Asura Math Helper Library
//
// Date : June 20, 2007
// Version : 3.0
// Author : Pocol
//------------------------------------------------------------------------------------------------------
#pragma endregion // File Description

#ifndef _ASURA_MATH_H_INCLUDED_
#define _ASURA_MATH_H_INCLUDED_ 

#pragma region Includes
//
// Includes
//
#include <iostream>
#include <cmath>
#include <vector>
#pragma endregion // Includes

#if _MSC_VER >= 1200
#pragma warning(push)
#endif
#pragma warning(disable:4201)

#pragma region Erros
//
// Errors
//

// defineŒ`ꂽmaxmin͎gȂB
// gĂꍇ̓G[bZ[W\
#if defined(min) || defined(max)
#error Error: min or max is #defined, probably in <windows.h>.  Put #define NOMINMAX before including windows.h to suppress windows global min,max macros.
#endif

#pragma endregion // Erros

#pragma region Defines
//
// Defines
//

// AsuraInline
#ifndef AsuraInline
#ifdef _MSC_VER
  #if (_MSC_VER >= 1200)
  #define AsuraInline __forceinline
  #else
  #define AsuraInline __inline
  #endif
#else
  #ifdef __cplusplus
  #define AsuraInline inline
  #else
  #define AsuraInline
  #endif
#endif
#endif

// AsuraTemplate
#ifndef AsuraTemplate
#define AsuraTemplate template<typename T>
#endif

// TYPEDEFINE_POINTER
#ifndef TYPEDEFINE_POINTER
#define TYPEDEFINE_POINTER( x )		typedef x*				P##x
#endif

// TYPEDEFINE_REFERENCE
#ifndef TYPEDEFINE_REFERENCE
#define TYPEDEFINE_REFERENCE( x )	typedef x&				R##x
#endif

// TYPEDEFINE_LIST
#ifndef TYPEDEFINE_LIST
#define TYPEDEFINE_LIST( x )				typedef std::vector<x>		x##List 
#endif

// TYPEDEFINE_PLIST
#ifndef TYPEDEFINE_PLIST
#define TYPEDEFINE_PLIST( x )			typedef std::vector<x*>	P##x##List
#endif

#pragma endregion // Defines

#pragma region Asura
//
// namespace Asura
//
namespace Asura
{

	#pragma region Math
	//
	// namespace Math
	//
	namespace Math
	{
		#pragma region Constant Number
		//
		// Constant Number
		//
		#ifndef ASURA_MATH_PI
		#define ASURA_MATH_PI
		const double Pi = 3.1415926535897932384626433832795;
		const double TwoPi = 6.283185307179586476925286766559;
		const double OneOverPi = 0.31830988618379067153776752674503;
		const double PiOver2 = 1.5707963267948966192313216916398;
		const double PiOver3 = 1.0471975511965977461542144610932;
		const double PiOver4 = 0.78539816339744830961566084581988;
		const double PiOver6 = 0.52359877559829887307710723054658;

		const float Pif = 3.141592654f;
		const float TwoPif = 6.283185307f;
		const float OneOverPif = 0.318309886f;
		const float PiOver2f = 1.570796326f;
		const float PiOver3f = 1.047197551f;
		const float PiOver4f = 0.785398163f;
		const float PiOver6f = 0.523598775f;
		#endif
		#pragma endregion // Constant Number

		#pragma region Forward Declarations
		//
		// Forward Declarations
		//
		AsuraTemplate class Vector2;
		AsuraTemplate class Vector3;
		AsuraTemplate class Vector4;
		AsuraTemplate class Matrix;
		AsuraTemplate class Plane;
		AsuraTemplate class Quaternion;
		AsuraTemplate class Color;
		#pragma endregion // Forward Declarations

		#pragma region Typedef
		//
		//@typedef
		//

		// uchar
		#ifndef _DEFINE_UCHAR_
		#define _DEFINE_UCHAR_ 1
		typedef unsigned char uchar;
		#endif

		// uint
		#ifndef _DEFINE_UINT_
		#define _DEFINE_UINT_ 1
		typedef unsigned int uint;
		#endif

		// ulong
		#ifndef _DEFINE_ULONG_
		#define _DEFINE_ULONG_ 1
		typedef unsigned long ulong;
		#endif

		// Int
		#ifndef _DEFINE_ASURA_INT_
		#define _DEFINE_ASURA_INT_ 1
		typedef int Int;
		#endif

		// Uint
		#ifndef _DEFINE_ASURA_UINT_
		#define _DEFINE_ASURA_UINT_ 1
		typedef uint Uint;
		#endif

		// Float
		#ifndef _DEFINE_ASURA_FLOAT_
		#define _DEFINE_ASURA_FLAOT_ 1
		typedef float Float;
		#endif

		// Double
		#ifndef _DEFINE_ASURA_DOUBLE_
		#define _DEFINE_ASURA_DOUBLE_ 1
		typedef double Double;
		#endif

		// Bool
		#ifndef _DEFINE_ASURA_BOOL_
		#define _DEFINE_ASURA_BOOL_ 1
		typedef bool Bool;
		#endif

		// List
		TYPEDEFINE_LIST( Int );
		TYPEDEFINE_LIST( Uint );
		TYPEDEFINE_LIST( Double );
		TYPEDEFINE_LIST( Float );
		TYPEDEFINE_LIST( Bool );
		TYPEDEFINE_PLIST( Int );
		TYPEDEFINE_PLIST( Uint );
		TYPEDEFINE_PLIST( Double );
		TYPEDEFINE_PLIST( Float );
		TYPEDEFINE_PLIST( Bool );

		// Vector, Matrix, Plane, Quaternion, Color
		typedef Vector2<int>			Vector2i;
		typedef Vector2<double>	Vector2d;
		typedef Vector2<float>		Vector2f;
		typedef Vector2<bool>		Vector2b;
		typedef Vector3<int>			Vector3i;
		typedef Vector3<double>	Vector3d;
		typedef Vector3<float>		Vector3f;
		typedef Vector3<bool>		Vector3b;
		typedef Vector4<int>			Vector4i;
		typedef Vector4<double>	Vector4d;
		typedef Vector4<float>		Vector4f;
		typedef Vector4<bool>		Vector4b;
		typedef Matrix<int>			Matrix4i;
		typedef Matrix<double>	Matrix4d;
		typedef Matrix<float>		Matrix4f;
		typedef Plane<int>				Planei;
		typedef Plane<double>		Planed;
		typedef Plane<float>			Planef;
		typedef Quaternion<int>		Quaternioni;
		typedef Quaternion<double>	Quaterniond;
		typedef Quaternion<float>	Quaternionf;
		typedef Color<float>			Color4f;
		typedef Color<double>		Color4d;

		// Pointer
		TYPEDEFINE_POINTER( Vector2i );
		TYPEDEFINE_POINTER( Vector2d );
		TYPEDEFINE_POINTER( Vector2f );
		TYPEDEFINE_POINTER( Vector2b );
		TYPEDEFINE_POINTER( Vector3i );
		TYPEDEFINE_POINTER( Vector3d );
		TYPEDEFINE_POINTER( Vector3f );
		TYPEDEFINE_POINTER( Vector3b );
		TYPEDEFINE_POINTER( Vector4i );
		TYPEDEFINE_POINTER( Vector4d );
		TYPEDEFINE_POINTER( Vector4f );
		TYPEDEFINE_POINTER( Vector4b );
		TYPEDEFINE_POINTER( Matrix4i );
		TYPEDEFINE_POINTER( Matrix4d );
		TYPEDEFINE_POINTER( Matrix4f );
		TYPEDEFINE_POINTER( Planei );
		TYPEDEFINE_POINTER( Planed );
		TYPEDEFINE_POINTER( Planef );
		TYPEDEFINE_POINTER( Quaternioni );
		TYPEDEFINE_POINTER( Quaterniond );
		TYPEDEFINE_POINTER( Quaternionf );
		TYPEDEFINE_POINTER( Color4f );
		TYPEDEFINE_POINTER( Color4d );

		// List
		TYPEDEFINE_LIST( Vector2i );
		TYPEDEFINE_LIST( Vector2d );
		TYPEDEFINE_LIST( Vector2f );
		TYPEDEFINE_LIST( Vector2b );
		TYPEDEFINE_LIST( Vector3i );
		TYPEDEFINE_LIST( Vector3d );
		TYPEDEFINE_LIST( Vector3f );
		TYPEDEFINE_LIST( Vector3b );
		TYPEDEFINE_LIST( Vector4i );
		TYPEDEFINE_LIST( Vector4d );
		TYPEDEFINE_LIST( Vector4f );
		TYPEDEFINE_LIST( Vector4b );
		TYPEDEFINE_LIST( Matrix4i );
		TYPEDEFINE_LIST( Matrix4d );
		TYPEDEFINE_LIST( Matrix4f );
		TYPEDEFINE_LIST( Planei );
		TYPEDEFINE_LIST( Planed );
		TYPEDEFINE_LIST( Planef );
		TYPEDEFINE_LIST( Quaternioni );
		TYPEDEFINE_LIST( Quaterniond );
		TYPEDEFINE_LIST( Quaternionf );
		TYPEDEFINE_LIST( Color4f );
		TYPEDEFINE_LIST( Color4d );

		// PointerList
		TYPEDEFINE_PLIST( Vector2i );
		TYPEDEFINE_PLIST( Vector2d );
		TYPEDEFINE_PLIST( Vector2f );
		TYPEDEFINE_PLIST( Vector2b );
		TYPEDEFINE_PLIST( Vector3i );
		TYPEDEFINE_PLIST( Vector3d );
		TYPEDEFINE_PLIST( Vector3f );
		TYPEDEFINE_PLIST( Vector3b );
		TYPEDEFINE_PLIST( Vector4i );
		TYPEDEFINE_PLIST( Vector4d );
		TYPEDEFINE_PLIST( Vector4f );
		TYPEDEFINE_PLIST( Vector4b );
		TYPEDEFINE_PLIST( Matrix4i );
		TYPEDEFINE_PLIST( Matrix4d );
		TYPEDEFINE_PLIST( Matrix4f );
		TYPEDEFINE_PLIST( Planei );
		TYPEDEFINE_PLIST( Planed );
		TYPEDEFINE_PLIST( Planef );
		TYPEDEFINE_PLIST( Quaternioni );
		TYPEDEFINE_PLIST( Quaterniond );
		TYPEDEFINE_PLIST( Quaternionf );
		TYPEDEFINE_PLIST( Color4f );
		TYPEDEFINE_PLIST( Color4d );
		#pragma endregion // Typdef

		#pragma region Global Functions
		//////////////////////////////////////////////////////////////////////
		//@@Function
		//////////////////////////////////////////////////////////////////////

		//--------------------------------------------------------------------------------------------
		// Name : max()
		// Desc : 2̒lrđ傫ق̒lԂ
		//--------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T max(T a, T b) { return (T)(a > b ? a : b); }

		//---------------------------------------------------------------------------------------------
		// Name : min()
		// Desc : 2̒lrďق̒lԂ
		//---------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T min(T a, T b) { return (T)(a < b ? a : b ); }

		//---------------------------------------------------------------------------------------------
		// Name : sign()
		// Desc : l0Ȃ0CvXȂ1.0C}CiXȂ-1.0Ԃ
		//---------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T sign(T x) { if(x==0.0) return (T)0.0; else if(x>0.0) return (T)1.0; return (T)-1.0; }

		//----------------------------------------------------------------------------------------------
		// Name : clamp()
		// Desc : x̒lab͈̔͂ɐ
		//----------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T clamp(T x, T a, T b) { return (T) Asura::Math::max(a, Asura::Math::min(b, x)); }

		//------------------------------------------------------------------------------------------------
		// Name : saturate()
		// Desc : l0`1͈̔͂ɐ
		//------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T saturate(T value) { return (T)clamp(value, (T)0, (T)1); }

		//------------------------------------------------------------------------------------------------
		// Name : lerp()
		// Desc : abp[^tŐ`Ԃ
		//------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T lerp(T a, T b, T t) { return ((T)1.0-t) * a + b * t; }

		//-------------------------------------------------------------------------------------------------
		// Name : ToDegree()
		// Desc : WAxɒlϊĕԂ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T ToDegree(T rad) { return (T)((double)(rad)*(180.0/Pi)); }

		//-------------------------------------------------------------------------------------------------
		// Name : ToRadian()
		// Desc : x烉WAɒlϊĕԂ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T ToRadian(T deg) { return (T)((double)(deg)*(Pi/180.0)); }

		//-------------------------------------------------------------------------------------------------
		// Name : sec()
		// Desc : x̃ZJg̒lvZĕԂ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T sec(T x) { return (T)(1.0/cos( (double)x ) ); }

		//-------------------------------------------------------------------------------------------------
		// Name : csc()
		// Desc : x̃RZJg̒lvZĕԂ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T csc(T x) { return (T)(1.0/sin( (double)x) ); }

		//-------------------------------------------------------------------------------------------------
		// Name : cot()
		// Desc : x̃R^WFg̒lvZĕԂ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T cot(T x) { return (T)(1.0/tan( (double)x) ); }

		//-------------------------------------------------------------------------------------------------
		// Name : asec()
		// Desc : x̃A[NZJg̒lvZĕԂ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T asec(T x) { return (T)(2.0*atan(1.0) - atan(Asura::sign( (double)x )/sqrt( (double)x*(double)x-1.0) ) ); }

		//-------------------------------------------------------------------------------------------------
		// Name : acsc()
		// Desc : x̃A[NRZJg̒lvZĕԂ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T acsc(T x) { return (T)(atan(Asura::sign( (double)x )/sqrt( (double)x*(double)x-1.0 ) ) ); }

		//-------------------------------------------------------------------------------------------------
		// Name : acot()
		// Desc : x̃A[NR^WFg̒lvZĕԂ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T acot(T x) { return (T)(2.0*atan(1.0) - atan( (double)x) ); }

		//--------------------------------------------------------------------------------------------------
		// Name : sech()
		// Desc : x̃nCp{bNZJg̒lvZĕԂ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T sech(T x) { return (T)(2.0/(exp( (double)x )+exp( (double)-x) ) ); }

		//--------------------------------------------------------------------------------------------------
		// Name : csch()
		// Desc : x̃nCp{bNRJg̒lvZĕԂ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T csch(T x) { return (T)(2.0/(exp( (double)x )-exp( (double)-x ) ) ); }

		//--------------------------------------------------------------------------------------------------
		// Name : coth()
		// Desc : x̃nCp{bN^WFg̒lvZĕԂ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T coth(T x) { return (T)( (exp( (double)x )+exp( (double)-x ))/(exp( (double)x )-exp( (double)-x ) ) ); }

		//--------------------------------------------------------------------------------------------------
		// Name : asinh()
		// Desc : x̃nCp{bNA[NTC̒lvZĕԂ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T asinh(T x) { return (T)(log( (double)x + sqrt( (double)x*(double)x + 1.0 ) ) ); }

		//--------------------------------------------------------------------------------------------------
		// Name : acosh()
		// Desc : x̃nCp{bNA[NRTC̒lvZĕԂ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T acosh(T x) { return (T)(log((double)x+sqrt((double)x*(double)x-1.0))); }

		//---------------------------------------------------------------------------------------------------
		// Name : atanh()
		// Desc : x̃nCp{bNA[N^WFg̒lvZĕԂ
		//---------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T atanh(T x) { return (T)(log((1.0+(double)x)/(1.0-(double)x))/2.0); }

		//---------------------------------------------------------------------------------------------------
		// Name : asech()
		// Desc : x̃nCp{bNA[NZJg̒lvZĕԂ
		//---------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T asech(T x) { return (T)(log((sqrt(-(double)x*(double)x+1.0)+1.0)/(double)x)); }

		//----------------------------------------------------------------------------------------------------
		// Name : acsch()
		// Desc : x̃nCp{bNA[NRZJg̒lvZĕԂ
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T acsch(T x) { return (T)(log((Asura::sign((double)x)*sqrt((double)x*(double)x+1.0)+1.0)/(double)x)); }

		//----------------------------------------------------------------------------------------------------
		// Name : acoth()
		// Desc : x̃nCp{bNA[NR^WFg̒lvZĕԂ
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate static AsuraInline T acoth(T x) { return (T)(log(((double)x+1.0)/(double)x-1.0)/2.0); }

		#pragma endregion // Global Functions

		#pragma region Vector2
		/////////////////////////////////////////////////////////////////////
		//@@Vector2 class
		/////////////////////////////////////////////////////////////////////
		AsuraTemplate
		class Vector2
		{
		public:
			union{ struct{ T x, y; }; T v[2]; };
			Vector2<T> (T nx=0, T ny=0) : x(nx), y(ny) {}
			Vector2<T> (const Vector2<T> &vec) : x(vec.x), y(vec.y) {}
			Vector2<T> (const Vector2<T> &a, const Vector2<T> &b) : x(a.x - b.x), y(a.y - b.y) {}
			Vector2<T> (const T *v) : x(v[0]), y(v[1]) {}
			operator T * ();
			operator const T* () const;
			Vector2<T> &operator= (const Vector2<T> &vec);
			Vector2<T> &operator= (const T &s);
			bool operator == (const Vector2<T> &vec)const;
			bool operator != (const Vector2<T> &vec)const;
			bool operator > (const Vector2<T> &vec)const;
			bool operator < (const Vector2<T> &vec)const;
			bool operator >= (const Vector2<T> &vec)const;
			bool operator <= (const Vector2<T> &vec)const;
			Vector2<T> operator+ (const Vector2<T> &vec)const;
			Vector2<T> operator- (const Vector2<T> &vec) const;
			Vector2<T> operator+() const;
			Vector2<T> operator-() const;
			Vector2<T> &operator+= (const Vector2<T> &vec);
			Vector2<T> &operator-= (const Vector2<T> &vec);
			Vector2<T> &operator*= (const T s);
			Vector2<T> &operator/= (const T s);
			Vector2<T> operator* (const T s)const;
			Vector2<T> operator/ (const T s)const;
			Vector2<T> operator* (const Vector2<T> &vec)const;
			Vector2<T> UnitVector() const;
			T Dot(const Vector2<T> &vec) const;
			T Length() const;
			T LengthSquared() const;
			void Normalize();
			Vector2<T> One();
			Vector2<T> Zero();
			Vector2<T> UnitX();	// (1.0, 0.0)
			Vector2<T> UnitY();	// (0.0, 1.0);
			void ConsoleOut();
			friend Vector2<T> operator * ( T s, const Vector2<T> &vec){ return Vector2<T>(s*vec.x, s*vec.y); }
			friend std::ostream& operator << (std::ostream &output, const Vector2<T> &vec)
			{ output << "(x, y) = (" << vec.x << ", " << vec.y << ")"; return output; }
		};
		#pragma endregion // Vector2

		#pragma region Vector3
		/////////////////////////////////////////////////////////////////////
		//@@Vector3 class
		/////////////////////////////////////////////////////////////////////
		AsuraTemplate
		class Vector3
		{
		public:
			union{ struct{ T x, y, z; }; T v[3]; };
			Vector3<T> (T nx=0.0, T ny=0.0, T nz=0.0) : x(nx), y(ny), z(nz) {}
			Vector3<T> (const Vector2<T> &vec, T nz) : x(vec.x), y(vec.y), z(nz) {}
			Vector3<T> (const Vector3<T> &vec) : x(vec.x), y(vec.y), z(vec.z) {}
			Vector3<T> (const Vector3<T> &a, const Vector3<T> &b) : x(a.x - b.x), y(a.y - b.y), z(a.z - b.z) {}
			Vector3<T> (const T *v) : x(v[0]), y(v[1]), z(v[2]) {}
			operator T * ();
			operator const T* () const;
			Vector3<T> &operator= (const Vector3<T> &vec);
			Vector3<T> &operator= (const T &s);
			bool operator== (const Vector3<T> &vec)const;
			bool operator!= (const Vector3<T> &vec)const;
			bool operator> (const Vector3<T> &vec)const;
			bool operator< (const Vector3<T> &vec)const;
			bool operator>= (const Vector3<T> &vec)const;
			bool operator<= (const Vector3<T> &vec)const;
			Vector3<T> operator+ (const Vector3<T> &vec)const;
			Vector3<T> operator+() const;
			Vector3<T> &operator+= (const Vector3<T> &vec);
			Vector3<T> operator- (const Vector3<T> &vec)const;
			Vector3<T> operator-() const;
			Vector3<T> &operator-= (const Vector3<T> &vec);
			Vector3<T> &operator*= (const T s);
			Vector3<T> &operator/= (const T s);
			Vector3<T> operator* (const T s)const;
			Vector3<T> operator/ (const T s)const;
			Vector3<T> operator* (const Vector3<T> &vec) const;
			Vector3<T> Cross(const Vector3<T> &vec) const; 
			Vector3<T> UnitVector() const; 
			T Dot(const Vector3<T> &vec) const;
			T Length() const;
			T LengthSquared() const;
			void Normalize();
			Vector2<T> xy();
			Vector3<T> One();
			Vector3<T> Zero();
			Vector3<T> UnitX();	// (1.0, 0.0, 0.0)
			Vector3<T> UnitY();	// (0.0, 1.0, 0.0)
			Vector3<T> UnitZ();	// (0.0, 0.0, 1.0)
			Vector3<T> Forward();
			Vector3<T> Backward();
			Vector3<T> Up();
			Vector3<T> Down();
			Vector3<T> Left();
			Vector3<T> Right();
			void ConsoleOut();
			Color<T> ToColor() { return Color<T>(x, y, z, 1.0); }
			friend Vector3<T> operator * ( T s, const Vector3<T> &vec) { return Vector3<T>(s*vec.x, s*vec.y, s*vec.z); }
			friend std::ostream& operator << (std::ostream &output, const Vector3<T> &vec)
			{ output << "(x, y, z) = (" << vec.x << ", " << vec.y << ", " << vec.z << ")"; return output; }
		};
		#pragma endregion // Vector3

		#pragma region Vector4
		/////////////////////////////////////////////////////////////////////
		//@@Vector4 class
		/////////////////////////////////////////////////////////////////////
		AsuraTemplate
		class Vector4
		{
		public:
			union{ struct{ T x, y, z, w; }; T v[4]; };
			Vector4<T> (T nx=0.0, T ny=0.0, T nz=0.0, T nw=0.0) : x(nx), y(ny), z(nz), w(nw) {}
			Vector4<T> (const Vector2<T> &vec, T nz, T nw) : x(vec.x), y(vec.y), z(nz), w(nw) {}
			Vector4<T> (const Vector3<T> &vec, T nw) : x(vec.x), y(vec.y), z(vec.z), w(nw) {}
			Vector4<T> (const Vector4<T> &vec) : x(vec.x), y(vec.y), z(vec.z), w(vec.w) {}
			Vector4<T> (const Vector4<T> &a, const Vector4<T> &b) : x(a.x - b.x), y(a.y - b.y), z(a.z - b.z), w(a.w - b.w) {}
			Vector4<T> (const T *v) : x(v[0]), y(v[1]), z(v[2]), w(v[3]) {}
			operator T * ();
			operator const T* () const;
			Vector4<T> &operator= (const Vector4<T> &vec);
			Vector4<T> &operator= (const T &s);
			bool operator== (const Vector4<T> &vec)const;
			bool operator!= (const Vector4<T> &vec)const;
			bool operator> (const Vector4<T> &vec)const;
			bool operator< (const Vector4<T> &vec)const;
			bool operator>= (const Vector4<T> &vec)const;
			bool operator <= (const Vector4<T> &vec)const;
			Vector4<T> operator+ (const Vector4<T> &vec)const;
			Vector4<T> operator+() const;
			Vector4<T> &operator+= (const Vector4<T> &vec);
			Vector4<T> operator- (const Vector4<T> &vec)const;
			Vector4<T> operator-() const;
			Vector4<T> &operator-= (const Vector4<T> &vec);
			Vector4<T> &operator*= (const T s);
			Vector4<T> &operator/= (const T s);
			Vector4<T> operator* (const T s)const;
			Vector4<T> operator/ (const T s)const;
			Vector4<T> operator* (const Vector4<T> &vec)const;
			Vector4<T> UnitVector() const;
			T Dot(const Vector4<T> &vec) const;
			T Length() const;
			T LengthSquared() const;
			void Normalize();
			Vector2<T> xy();
			Vector3<T> xyz();
			Vector4<T> One();
			Vector4<T> Zero();
			Vector4<T> UnitX();	// (1.0, 0.0, 0.0, 0.0)
			Vector4<T> UnitY();	// (0.0, 1.0, 0.0, 0.0)
			Vector4<T> UnitZ();	// (0.0, 0.0, 1.0, 0.0)
			Vector4<T> UnitW();	// (0.0, 0.0, 0.0, 1.0)
			void ConsoleOut();
			Color<T> ToColor() { return Color<T>(x, y, z, w); }
			friend Vector4<T> operator * ( T s, const Vector4<T> &vec) { return Vector4<T>(s*vec.x, s*vec.y, s*vec.z, s*vec.w); }
			friend std::ostream& operator << (std::ostream &output, const Vector4<T> &vec)
			{ output << "(x, y, z, w) = (" << vec.x << ", " << vec.y << ", " << vec.z << ", " << vec.w << ")"; return output; }
		};
		#pragma endregion // Vector4

		#pragma region Vector Functions
		//////////////////////////////////////////////////////////////////////
		//@@Vector function
		/////////////////////////////////////////////////////////////////////

		//--------------------------------------------------------------------------------------------------
		// Name : Abs()
		// Desc : e̐ΒlƂ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Abs(Vector2<T> v);
		AsuraTemplate Vector3<T> Abs(Vector3<T> v);
		AsuraTemplate Vector4<T> Abs(Vector4<T> v);
		AsuraTemplate void Abs(Vector2<T> &v, Vector2<T> &result);
		AsuraTemplate void Abs(Vector3<T> &v, Vector3<T> &result);
		AsuraTemplate void Abs(Vector4<T> &v, Vector4<T> &result);

		//-------------------------------------------------------------------------------------------------
		// Name : Clamp()
		// Desc : v̒lminimaxi͈̔͂ɐ
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Clamp(Vector2<T> v, Vector2<T> min, Vector2<T> max);
		AsuraTemplate Vector3<T> Clamp(Vector3<T> v, Vector3<T> min, Vector3<T> max);
		AsuraTemplate Vector4<T> Clamp(Vector4<T> v, Vector4<T> min, Vector4<T> max);
		AsuraTemplate void Clamp(Vector2<T> &v, Vector2<T> &min, Vector2<T> &max, Vector2<T> &result);
		AsuraTemplate void Clamp(Vector3<T> &v, Vector3<T> &min, Vector3<T> &max, Vector3<T> &result);
		AsuraTemplate void Clamp(Vector4<T> &v, Vector4<T> &min, Vector4<T> &max, Vector4<T> &result);

		//-------------------------------------------------------------------------------------------------
		// Name : Distance()
		// Desc : v1v2̊Ԃ̋߂
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate T Distance(Vector2<T> v1, Vector2<T> v2);
		AsuraTemplate T Distance(Vector3<T> v1, Vector3<T> v2);
		AsuraTemplate T Distance(Vector4<T> v1, Vector4<T> v2);
		AsuraTemplate void Distance(Vector2<T> &v1, Vector2<T> &v2, T &result);
		AsuraTemplate void Distance(Vector3<T> &v1, Vector3<T> &v2, T &result);
		AsuraTemplate void Distance(Vector4<T> &V1, Vector4<T> &v2, T &result);

		//-------------------------------------------------------------------------------------------------
		// Name : DistanceSquared()
		// Desc : v1v2̊Ԃ̋2߂
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate T DistanceSquared(Vector2<T> v1, Vector2<T> v2);
		AsuraTemplate T DistanceSquared(Vector3<T> v1, Vector3<T> v2);
		AsuraTemplate T DistanceSquared(Vector4<T> v1, Vector4<T> v2);
		AsuraTemplate void DistanceSquared(Vector2<T> &v1, Vector2<T> &v2, T &result);
		AsuraTemplate void DistanceSquared(Vector3<T> &v1, Vector3<T> &v2, T &reslut);
		AsuraTemplate void DistanceSquared(Vector4<T> &v1, Vector4<T> &v2, T &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Length()
		// Desc : v1v2̊Ԃ̋߂iDistance֐Ăяoj
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate T Length(Vector2<T> v1, Vector2<T> v2);
		AsuraTemplate T Length(Vector3<T> v1, Vector2<T> v2);
		AsuraTemplate T Length(Vector3<T> v1, Vector2<T> v2);
		AsuraTemplate void Length(Vector2<T> &v1, Vector2<T> &v2, T &result);
		AsuraTemplate void Length(Vector3<T> &v1, Vector3<T> &v2, T &result);
		AsuraTemplate void Length(Vector4<T> &v1, Vector4<T> &v2, T &result);

		//--------------------------------------------------------------------------------------------------
		// Name : LengthSquared()
		// Desc : v1v2̊Ԃ̋2߂iDistanceSquared֐Ăяoj
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate T LengthSquared(Vector2<T> v1, Vector2<T> v2);
		AsuraTemplate T LengthSquared(Vector3<T> v1, Vector3<T> V2);
		AsuraTemplate T LengthSquared(Vector4<T> v1, Vector4<T> v2);
		AsuraTemplate void LengthSquared(Vector2<T> &v1, Vector2<T> &v2, T &result);
		AsuraTemplate void LengthSquraed(Vector3<T> &v1, Vector3<T> &v2, T &result);
		AsuraTemplate void LengthSquared(Vector4<T> &v1, Vector4<T> &v2, T &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Dot()
		// Desc : v1v2̓ς߂
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate T Dot(Vector2<T> v1, Vector2<T> v2);
		AsuraTemplate T Dot(Vector3<T> v1, Vector3<T> v2);
		AsuraTemplate T Dot(Vector4<T> v1, Vector4<T> v2);
		AsuraTemplate void Dot(Vector2<T> &v1, Vector2<T> &v2, T &result);
		AsuraTemplate void Dot(Vector3<T> &v1, Vector3<T> &v2, T &result);
		AsuraTemplate void Dot(Vector4<T> &v1, Vector4<T> &v2, T &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Cross()
		// Desc : v1v2̊Oς߂
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector3<T> Cross(Vector3<T> v1, Vector3<T> v2);
		AsuraTemplate void Cross(Vector3<T> &v1, Vector3<T> &v2, Vector3<T> &result);
			
		//--------------------------------------------------------------------------------------------------
		// Name : Normalize()
		// Desc : v𐳋K
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Normalize(Vector2<T> v);
		AsuraTemplate Vector3<T> Normalize(Vector3<T> v);
		AsuraTemplate Vector4<T> Normalize(Vector4<T> v);
		AsuraTemplate void Normalize(Vector2<T> &v, Vector2<T> &result);
		AsuraTemplate void Normalize(Vector3<T> &v, Vector3<T> &result);
		AsuraTemplate void Normalize(Vector4<T> &v, Vector4<T> &result);

		//-------------------------------------------------------------------------------------------------
		// Name : GetNormal()
		// Desc : v1, v2, v3ȂOp`̖@xNg߂
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate Vector3<T> GetNormal(Vector3<T> v1, Vector3<T> v2, Vector3<T> v3);
		AsuraTemplate void GetNormal(Vector3<T> &v1, Vector3<T> &v2, Vector3<T> &v3, Vector3<T> &result);

		//-------------------------------------------------------------------------------------------------
		// Name : GetQuadNormal()
		// Desc : v1, v2, v3, v4Ȃlp`̖@xNg߂
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate Vector3<T> GetQuadNormal(Vector3<T> v1, Vector3<T> v2, Vector3<T> v3, Vector3<T> v4);
		AsuraTemplate void GetQuadNormal(Vector3<T> &v1, Vector3<T> &v2, Vector3<T> &v3, Vector3<T> &v4, Vector3<T> &result);

		//-------------------------------------------------------------------------------------------------
		// Name : GetTangent()
		// Desc : _WƃeNX`WڃxNg߂
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate Vector3<T> GetTangent(Vector3<T> v1, Vector3<T> v2, Vector3<T> v3, Vector2<T> t1, Vector2<T> t2, Vector2<T> t3 );
		AsuraTemplate void GetTangent(Vector3<T> &v1, Vector3<T> &v2, Vector3<T> &v3, Vector2<T> &t1, Vector2<T> &t2, Vector2<T> &t3, Vector3<T> &result);

		//-------------------------------------------------------------------------------------------------
		// Name : GetBinormal()
		// Desc : @xNgƐڃxNg]@xNg߂
		//-------------------------------------------------------------------------------------------------
		AsuraTemplate Vector3<T> GetBinormal(Vector3<T> normal, Vector3<T> tangent);
		AsuraTemplate void GetBinormal(Vector3<T> &normal, Vector3<T> &tangent, Vector3<T> &result);

		//--------------------------------------------------------------------------------------------------
		// Name : GetCrossingAngle()
		// Desc : v1v2̊Ԃ̂Ȃp߂
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate T GetCrossingAngle(Vector2<T> v1, Vector2<T> v2);
		AsuraTemplate T GetCrossingAngle(Vector3<T> v1, Vector3<T> v2);
		AsuraTemplate T GetCrossingAngle(Vector4<T> v1, Vector4<T> v2);
		AsuraTemplate void GetCrossingAngle(Vector2<T> &v1, Vector2<T> &v2, T &result);
		AsuraTemplate void GetCrossingAngle(Vector3<T> &v1, Vector3<T> &v2, T &result);
		AsuraTemplate void GetCrossingAngle(Vector4<T> &v1, Vector4<T> &v2, T &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Min()
		// Desc : v1v2̐re̍ŏlxNgƂďo͂
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Min(Vector2<T> v1, Vector2<T> v2);
		AsuraTemplate Vector3<T> Min(Vector3<T> v1, Vector3<T> v2);
		AsuraTemplate Vector4<T> Min(Vector4<T> v1, Vector4<T> v2);
		AsuraTemplate void Min(Vector2<T> &v1, Vector2<T> &v2, Vector2<T> &result);
		AsuraTemplate void Min(Vector3<T> &v1, Vector3<T> &v2, Vector3<T> &result);
		AsuraTemplate void Min(Vector4<T> &v1, Vector4<T> &v2, Vector4<T> &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Max()
		// Desc : v1v2̐re̍őlxNgƂďo͂
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Max(Vector2<T> v1, Vector2<T> v2);
		AsuraTemplate Vector3<T> Max(Vector3<T> v1, Vector3<T> v2);
		AsuraTemplate Vector4<T> Max(Vector4<T> v1, Vector4<T> v2);
		AsuraTemplate void Max(Vector2<T> &v1, Vector2<T> &v2, Vector2<T> &result);
		AsuraTemplate void Max(Vector3<T> &v1, Vector3<T> &v2, Vector3<T> &result);
		AsuraTemplate void Max(Vector4<T> &v1, Vector4<T> &v2, Vector4<T> &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Reflect()
		// Desc : ˕iCʖ@nƂCvZɂċ߂ꂽ˃xNgԂ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Reflect(Vector2<T> i, Vector2<T> n);
		AsuraTemplate Vector3<T> Reflect(Vector3<T> i, Vector3<T> n);
		AsuraTemplate void Reflect(Vector2<T> &i, Vector2<T> &n, Vector2<T> &result);
		AsuraTemplate void Reflect(Vector3<T> &i, Vector3<T> &n, Vector3<T> &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Refract()
		// Desc : ˕iCʖ@nC܌WetaƂCvZɂċ߂ꂽ܃xNgԂ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector3<T> Refract(Vector3<T> i, Vector3<T> n, Vector3<T> eta);
		AsuraTemplate void Refract(Vector3<T> &i, Vector3<T> &n, Vector3<T> &eta, Vector3<T> &result);

		//---------------------------------------------------------------------------------------------------
		// Name : Barycentric()
		// Desc : w肵xNggāCdSWł̓_Ԃ
		// Param : f, g@dW
		//---------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Barycentric(Vector2<T> v1, Vector2<T> v2, Vector2<T> v3, T f, T g);
		AsuraTemplate Vector3<T> Barycentric(Vector3<T> v1, Vector3<T> v2, Vector3<T> v3, T f, T g);
		AsuraTemplate Vector4<T> Barycentric(Vector4<T> v1, Vector4<T> v2, Vector4<T> v3, T f, T g);
		AsuraTemplate void Barycentric(Vector2<T> &v1, Vector2<T> &v2, Vector2<T> &v3, T f, T g, Vector2<T> &result);
		AsuraTemplate void Barycentric(Vector3<T> &v1, Vector3<T> &v2, Vector3<T> &v3, T f, T g, Vector3<T> &result);
		AsuraTemplate void Barycentric(Vector4<T> &v1, Vector4<T> &v2, Vector4<T> &v3, T f, T g, Vector4<T> &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Lerp()
		// Desc : `Ԃs
		// Param : t@dW@
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Lerp(Vector2<T> v1, Vector2<T> v2, T t);
		AsuraTemplate Vector3<T> Lerp(Vector3<T> v1, Vector3<T> v2, T t);
		AsuraTemplate Vector4<T> Lerp(Vector4<T> v1, Vector4<T> v2, T t);
		AsuraTemplate void Lerp(Vector2<T> &v1, Vector2<T> &v2, T t, Vector2<T> &result);
		AsuraTemplate void Lerp(Vector3<T> &v1, Vector3<T> &v2, T t, Vector3<T> &result);
		AsuraTemplate void Lerp(Vector4<T> &v1, Vector4<T> &v2, T t, Vector4<T> &result);

		//---------------------------------------------------------------------------------------------------
		// Name : Hermite()
		// Desc : w肳ꂽxNggăG~[g̃XvCԂs
		// Param : v1, v2@ʒuxNg
		//             t1, t2@ڐxNg
		//@@@s@dW
		//---------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Hermite(
			Vector2<T> v1, Vector2<T> t1,
			Vector2<T> v2, Vector2<T> t2, T s);
		AsuraTemplate Vector3<T> Hermite(
			Vector3<T> v1, Vector3<T> t1,
			Vector3<T> v2, Vector3<T> t2, T s);
		AsuraTemplate Vector4<T> Hermite(
			Vector4<T> v1, Vector4<T> t1,
			Vector4<T> v2, Vector4<T> t2, T s);	
		AsuraTemplate void Hermite(
		   Vector2<T> &v1, Vector2<T> &t1,
		   Vector2<T> &v2, Vector2<T> &t2, T s, Vector2<T> &result);
		AsuraTemplate void Hermite(
		   Vector3<T> &v1, Vector3<T> &t1,
		   Vector3<T> &v2, Vector3<T> &t2, T s, Vector3<T> &result);
		AsuraTemplate void Hermite(
			Vector4<T> &v1, Vector4<T> &t1,
			Vector4<T> &v2, Vector4<T> &t2, T s, Vector4<T> &result);

		//----------------------------------------------------------------------------------------------------
		// Name : CatmullRom()
		// Desc : w肳ꂽxNggCatmull-RomԂs
		// Param : v1, v2, v3, v4@ʒuxNg
		//             s@dW
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> CatmullRom(
			Vector2<T> v0, Vector2<T> v1,
			Vector2<T> v2, Vector2<T> v3, T s);
		AsuraTemplate Vector3<T> CatmullRom(
			Vector3<T> v0, Vector3<T> v1,
			Vector3<T> v2, Vector3<T> v3, T s);
		AsuraTemplate Vector4<T> CatmullRom(
			Vector4<T> v0, Vector4<T> v1,
			Vector4<T> v2, Vector4<T> v3, T s);
		AsuraTemplate void CatmullRom(
			Vector2<T> &v0, Vector2<T> &v1,
			Vector2<T> &v2, Vector2<T> &v3, T s, Vector2<T> &result);
		AsuraTemplate void CatmullRom(
			Vector3<T> &v0, Vector3<T> &v1,
			Vector3<T> &v2, Vector3<T> &v3, T s, Vector3<T> &result);
		AsuraTemplate void CatmullRom(
			Vector4<T> &v0, Vector4<T> &v1,
			Vector4<T> &v2, Vector4<T> &v3, T s, Vector4<T> &result);

		//----------------------------------------------------------------------------------------------------
		// Name : Bezier()
		// Desc : w肳ꂽxNggăxWGԂs
		// Param : v0, v1, v2, v3@_̈ʒuxNg
		//             t@dW
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Bezier(
			Vector2<T> v0, Vector2<T> v1,
			Vector2<T> v2, Vector2<T> v3, T t);
		AsuraTemplate Vector3<T> Bezier(
			Vector3<T> v0, Vector3<T> v1,
			Vector3<T> v2, Vector3<T> v3, T t);
		AsuraTemplate Vector4<T> Bezier(
			Vector4<T> v0, Vector4<T> v1,
			Vector4<T> v2, Vector4<T> v3, T t);
		AsuraTemplate void Bezier(	
			Vector2<T> &v0, Vector2<T> &v1,
			Vector2<T> &v2, Vector2<T> &v3, T t, Vector2<T> &result);
		AsuraTemplate void Bezier(
			Vector3<T> &v0, Vector3<T> &v1,
			Vector3<T> &v2, Vector3<T> &v3, T t, Vector3<T> &result);
		AsuraTemplate void Bezier(
			Vector4<T> &v0, Vector4<T> &v1,
			Vector4<T> &v2, Vector4<T> &v3, T t, Vector4<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : B_Spline()
		// Desc : w肳ꂽxNggBXvCԂs
		// Param : v0, v1, v2, v3@ʒuxNg
		//             t@dW
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> B_Spline(
			Vector2<T> v0, Vector2<T> v1,
			Vector2<T> v2, Vector2<T> v3, T t);
		AsuraTemplate Vector3<T> B_Spline(
			Vector3<T> v0, Vector3<T> v1,
			Vector3<T> v2, Vector3<T> v3, T t);
		AsuraTemplate Vector4<T> B_Spline(
			Vector4<T> v0, Vector4<T> v1,
			Vector4<T> v2, Vector4<T> v3, T t);
		AsuraTemplate void B_Spline(
			Vector2<T> &v0, Vector2<T> &v1,
			Vector2<T> &v2, Vector2<T> &v3, T t, Vector2<T> &result);
		AsuraTemplate void B_Spline(
			Vector3<T> &v0, Vector3<T> &v1,
			Vector3<T> &v2, Vector3<T> &v3, T t, Vector3<T> &result);
		AsuraTemplate void B_Spline(
			Vector4<T> &v0, Vector4<T> &v1,
			Vector4<T> &v2, Vector4<T> &v3, T t, Vector4<T> &result);

		//--------------------------------------------------------------------------------------------------
		// Name : SmoothStep()
		// Desc :
		//---------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> SmoothStep(Vector2<T> value1, Vector2<T> value2, T amount);
		AsuraTemplate Vector3<T> SmoothStep(Vector3<T> value1, Vector3<T> value2, T amount);
		AsuraTemplate Vector4<T> SmoothStep(Vector4<T> value1, Vector4<T> value2, T amount);
		AsuraTemplate void SmoothStep(Vector2<T> &value1, Vector2<T> &value2, T amount, Vector2<T> &result);
		AsuraTemplate void SmoothStep(Vector3<T> &value1, Vector3<T> &value2, T amount, Vector3<T> &result);
		AsuraTemplate void SmoothStep(Vector4<T> &value1, Vector4<T> &value2, T amount, Vector4<T> &result);

		//--------------------------------------------------------------------------------------------------
		// Name : Transform()
		// Desc : xNgsŕϊ
		//--------------------------------------------------------------------------------------------------
		AsuraTemplate Vector2<T> Transform(Vector2<T> position, Matrix<T> matrix);
		AsuraTemplate Vector3<T> Transform(Vector3<T> position, Matrix<T> matrix);
		AsuraTemplate Vector4<T> Transform(Vector4<T> position, Matrix<T> matrix);
		AsuraTemplate void Transform(Vector2<T> &position, Matrix<T> &matrix, Vector2<T> &result);
		AsuraTemplate void Transform(Vector3<T> &position, Matrix<T> &matrix, Vector3<T> &result);
		AsuraTemplate void Transform(Vector4<T> &position, Matrix<T> &matrix, Vector4<T> &result);
		AsuraTemplate Vector2<T> TransformNormal(Vector2<T> normal, Matrix<T> matrix);
		AsuraTemplate Vector3<T> TransformNormal(Vector3<T> normal, Matrix<T> matrix);
		AsuraTemplate void TransformNormal(Vector2<T> &normal, Matrix<T> &matrix, Vector2<T> &result);
		AsuraTemplate void TransformNormal(Vector3<T> &normal, Matrix<T> &matrix, Vector3<T> &result);

		#pragma endregion // Vector Functions

		#pragma region Matrix 
		//////////////////////////////////////////////////////////////////////
		// Matrix class
		//////////////////////////////////////////////////////////////////////
		AsuraTemplate 
		class Matrix
		{
		public:
			union 
			{
				struct 
				{
					T	_11, _12, _13, _14;
					T	_21, _22, _23, _24;
					T	_31, _32, _33, _34;
					T	_41, _42, _43, _44;
				};
				T m[4][4];
			};

			Matrix();
			Matrix(const T* m);
			Matrix(const Matrix<T> &mat);
			Matrix(T m11, T m12, T m13, T m14,
				T m21, T m22, T m23, T m24,
				T m31, T m32, T m33, T m34,
				T m41, T m42, T m43, T m44 );
			T& operator() (unsigned int row, unsigned int col);
			T operator() (unsigned int row, unsigned int col) const;
			operator T*();
			operator const T*() const;
			Matrix<T>& operator = (const Matrix<T> &mat);
			Matrix<T>& operator *= (const Matrix<T> &mat);
			Matrix<T>& operator += (const Matrix<T> &mat);
			Matrix<T>& operator -= (const Matrix<T> &mat);
			Matrix<T>& operator *= (T t);
			Matrix<T>& operator /= (T t);
			Matrix<T> operator + () const;
			Matrix<T> operator - () const;
			Matrix<T> operator * (const Matrix<T> &mat) const;
			Matrix<T> operator + (const Matrix<T> &mat) const;
			Matrix<T> operator - (const Matrix<T> &mat) const;
			Matrix<T> operator * (T t) const;
			Matrix<T> operator / (T t) const;
			Vector4<T> operator* (const Vector4<T> &vec) const;
			bool operator == (const Matrix<T> &mat) const;
			bool operator !=  (const Matrix<T> &mat) const;
			Matrix<T> Identity();
			Matrix<T> Zero();
			Matrix<T> One();
			T Determinant();

			Vector4<T> _11_12_13_14();
			Vector4<T> _21_22_23_24();
			Vector4<T> _31_32_33_34();
			Vector4<T> _41_42_43_44();

			Vector4<T> _11_21_31_41();
			Vector4<T> _12_22_32_42();
			Vector4<T> _13_23_33_43();
			Vector4<T> _14_24_34_44();
			
			void ConsoleOut();

			friend Matrix<T> operator *(T s, const Matrix<T> &mat)
			{
				return Matrix<T>(
					s*mat._11, s*mat._12, s*mat._13, s*mat._14,
					s*mat._21, s*mat._22, s*mat._23, s*mat._24,
					s*mat._31, s*mat._32, s*mat._33, s*mat._34,
					s*mat._41, s*mat._42, s*mat._43, s*mat._44);
			}
		};
		#pragma endregion // Matrix

		#pragma region Matrix Functions
		//////////////////////////////////////////////////////////////////////////
		// Matrix Functions
		//////////////////////////////////////////////////////////////////////////

		//-----------------------------------------------------------------------------------------------------
		// Name : Transpose()
		// Desc : s]u
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> Transpose(Matrix<T> value);
		AsuraTemplate void Transpose(Matrix<T> &value, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : Multiply()
		// Desc : s̐ςvZ
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> Multiply(Matrix<T> value1, Matrix<T> value2);
		AsuraTemplate void Multiply(Matrix<T> &value1, Matrix<T> &value2, Matrix<T> &result);
		AsuraTemplate Matrix<T> Multiply(Matrix<T> value, T scaleFactor);
		AsuraTemplate void Multiply(Matrix<T> &value, T scaleFactor, Matrix<T> &result);

		//------------------------------------------------------------------------------------------------------
		// Name : MultiplyTranspose()
		// Desc : 2̍s̐ς̓]uvZ
		//------------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> MultiplyTranspose(Matrix<T> value1, Matrix<T> value2);
		AsuraTemplate void MultiplyTranspose(Matrix<T> &value1, Matrix<T> &value2, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : Invert()
		// Desc : tsvZ
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> Invert(Matrix<T> value);
		AsuraTemplate void Invert(Matrix<T> &value, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateFromQuaternion()
		// Desc : ls쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateFromQuaternion(Quaternion<T> qua);
		AsuraTemplate void CreateFromQuaternion(Quaternion<T> &qua, Matrix<T> &result);
			
		//-----------------------------------------------------------------------------------------------------
		// Name : CreateScale()
		// Desc : gs쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateScale(T scale);
		AsuraTemplate void CreateSacle(T scale, Matrix<T> &result);
		AsuraTemplate Matrix<T> CreateScale(T xScale, T yScale, T zScale);
		AsuraTemplate void CreateScale(T xScale, T yScale, T zScale, Matrix<T> &result);
		AsuraTemplate Matrix<T> CreateScale(Vector3<T> scales);
		AsuraTemplate void CreateScale(Vector3<T> &scales, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateTranslation()
		// Desc : ItZbgw肵čs쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateTranslation(T xPosition, T yPosition, T zPosition);
		AsuraTemplate void CreateTranslation(T xPosition, T yPosition, T zPosition, Matrix<T> &result);
		AsuraTemplate Matrix<T> CreateTranslation(Vector3<T> position);
		AsuraTemplate void CreateTranslation(Vector3<T> &position, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateRotationX()
		// Desc : x]ɂĉ]s쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateRotationX(T radians);
		AsuraTemplate void CreateRotationX(T radians, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateRoataionY()
		// Desc : y]ɂĉ]s쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateRotationY(T radians);
		AsuraTemplate void CreateRotationY(T radians, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateRotationZ()
		// Desc : z]ɂĉ]s쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateRotationZ(T radians);
		AsuraTemplate void CreateRotationZ(T radians, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateFromAxisAngle()
		// Desc : Cӎ]ɂĉ]s쐬
		// Param : angle@]px(WAP)
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateFromAxisAngle(Vector3<T> axis, T radians);
		AsuraTemplate void CreateFromAxisAngle(Vector3<T> &axis, T radians, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateLookAt()
		// Desc : EWnr[s쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateLookAt(Vector3<T> cameraPosition, Vector3<T> cameraTarget, Vector3<T> caneraUpVector);
		AsuraTemplate void CreateLookAt(Vector3<T> &cameraPosition, Vector3<T> &cameraTarget, Vector3<T> &cameraUpVector, Vector3<T> &result);

		//----------------------------------------------------------------------------------------------------
		// Name : CreatePerspective()
		// Desc : EWnߎˉes쐬
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreatePerspective(T width, T height, T nearPlaneDistance, T farPlaneDistance);
		AsuraTemplate void CreatePerspective(T width, T height, T nearPlaneDistance, T farPlaneDistance, Matrix<T> &result);

		//----------------------------------------------------------------------------------------------------
		// Name : CreatePerspectiveFiledOfView()
		// Desc : ɊÂĉEWnߎˉes쐬
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreatePerspectiveFieldOfView(T fieldOfView, T aspectRatio, T nearPlaneDistance, T farPlaneDistance);
		AsuraTemplate void CreatePerspectiveFieldOfView(T fieldOfView, T apectRatio, T nearPlaneDistance, T farPlaneDistance, Matrix<T> &result);

		//----------------------------------------------------------------------------------------------------
		// Name : CreatePerspectiveOffcenter()
		// Desc : JX^}CYEWnߎˉes쐬
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreatePerspectiveOffcenter(T left, T right, T bottom, T top, T nearPlaneDistance, T farPlaneDistance);
		AsuraTemplate void CreatePerspectiveOffcenter(T left, T right, T bottom, T top, T nearPlaenDistance, T farPlaenDistance, Matrix<T> &result);

		//----------------------------------------------------------------------------------------------------
		// Name : CreateOrthographic()
		// Desc : EWnˉes쐬
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateOrthographic(T width, T height, T zNearPlane, T zFarPlane);
		AsuraTemplate void CreateOrthographic(T width, T height, T zNearPlane, T zFarPlane, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateOrthograhicOffcenter()
		// Desc : JX^}CYEWnˉes쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateOrthographicOffcenter(T left, T right, T bottom, T top, T zNearPlane, T zFarPlane);
		AsuraTemplate void CreateOrthographicOffcenter(T left, T right, T bottom, T top, T zNearPlane, T zFarPlane, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateShadow()
		// Desc : WIg𕽖ʂɎˉes쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateShadow(Vector3<T> lightDirection, Plane<T> plane);
		AsuraTemplate void CreateShadow(Vector3<T> &lightDirection, Plane<T> &plane, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : CreateReflect()
		// Desc : ʂɑ΂čWn𔽓]s쐬
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> CreateReflection(Plane<T> plane);
		AsuraTemplate void CreateReflection(Plane<T> &plane, Matrix<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : Lerp()
		// Desc : 2̍s`Ԃ
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Matrix<T> Lerp(Matrix<T> value1, Matrix<T> value2, T amount);
		AsuraTemplate void Lerp(Matrix<T> &value1, Matrix<T> &value2, T amount, Matrix<T> &result);

		#pragma endregion // Matrix Functions

		#pragma region Plane
		////////////////////////////////////////////////////////////////////////////
		//@@Plane
		////////////////////////////////////////////////////////////////////////////
		AsuraTemplate 
		class Plane
		{
		public:
			T a, b, c, d;
			Plane<T>(T na=0.0, T nb=0.0, T nc=0.0, T nd=0.0) : a(na), b(nb), c(nc), d(nd) {}
			Plane<T>(Vector3<T> normal, T nd) : a(normal.x), b(normal.y), c(normal.z), d(nd) {}
			Plane<T>(Vector3<T> position1, Vector3<T> position2, Vector3<T> position3);
			Plane<T>(Vector4<T> value ) : a(value.x), b(value.y), c(value.z), d(value.w) {}
			Plane<T>(const T *pf);
			operator T* ();
			operator const T*() const;
			Plane<T>& operator = (const Plane<T> &p);
			Plane<T>& operator *= (T f);
			Plane<T>& operator /= (T f);
			Plane<T> operator + () const;
			Plane<T> operator - () const;
			Plane<T> operator * (T f)const;
			Plane<T> operator / (T f)const;
			friend Plane<T> operator * ( T , const Plane<T> &);
			bool operator == (const Plane<T> &) const;
			bool operator != (const Plane<T> & ) const;
			Vector3<T> abc();
			Vector4<T> ToVector4();
			void Normalize();
			Plane<T> Zero();
			Plane<T> One();
			void ConsoleOut();
			friend std::ostream& operator << (std::ostream &output, const Plane<T> &plane)
			{ output << "(a, b, c, d) = (" << plane.a << ", " << plane.b << ", " << plane.c << ", " << plane.d << ")"; return output; }
		};
		#pragma endregion // Plane

		#pragma region Plane Functions
		////////////////////////////////////////////////////////////////////////////
		// Plane Function
		////////////////////////////////////////////////////////////////////////////

		//--------------------------------------------------------------------------------------------------------
		// Name : Dot()
		// Desc : ʂ4xNg̓ςvZ
		//--------------------------------------------------------------------------------------------------------
		AsuraTemplate T Dot(Plane<T> p, Vector4<T> v);
		AsuraTemplate void Dot(Plane<T> &p, Vector4<T> &v, T &result);

		//---------------------------------------------------------------------------------------------------------
		// Name : DotCoordinate()
		// Desc : ʂ3xNg̓ςvZBw̒l1Ƃ݂Ȃ
		//---------------------------------------------------------------------------------------------------------
		AsuraTemplate T DotCoordinate(Plane<T> p, Vector3<T> v);
		AsuraTemplate void DotCoordinate(Plane<T> &p, Vector3<T> &v, T &result);

		//--------------------------------------------------------------------------------------------------------
		// Name : DotNormal()
		// Desc : ʂ3xNg̓ςvZBw̒l0Ƃ݂Ȃ
		//--------------------------------------------------------------------------------------------------------
		AsuraTemplate T DotNormal(Plane<T> p, Vector3<T> v);
		AsuraTemplate void DotNormal(Plane<T> &p, Vector3<T> &v, T &result);

		//-------------------------------------------------------------------------------------------------------
		// Name : Normalize()
		// Desc : ʂ̌W𐳋KāCʂ̖@̒𐳋K
		//-------------------------------------------------------------------------------------------------------
		AsuraTemplate Plane<T> Normalize(Plane<T> p);
		AsuraTemplate void Normalize(Plane<T> &p, Plane<T> &result);

		#pragma endregion // Plane Functions

		#pragma region Quaternion
		////////////////////////////////////////////////////////////////////////////
		// Quaternion
		////////////////////////////////////////////////////////////////////////////
		AsuraTemplate 
		class Quaternion
		{
		public:
			union { struct { T x, y, z, w; }; T v[4]; };
			Quaternion<T>(T nx=0.0, T ny=0.0, T nz=0.0, T nw=0.0 ) : x(nx), y(ny), z(nz), w(nw) {}
			Quaternion<T>(Vector3<T> value, T nw) : x(value.x), y(value.y), z(value.z), w(nw) {}
			Quaternion<T>(const T* qua);
			operator T* ();
			operator const T*() const;
			Quaternion<T>& operator = (const Quaternion<T> &qua);
			Quaternion<T>& operator += (const Quaternion<T> &qua);
			Quaternion<T>& operator -= (const Quaternion<T> &qua);
			Quaternion<T>& operator *= (const Quaternion<T> &qua);
			Quaternion<T>& operator *= (T t);
			Quaternion<T>& operator /= (T t);
			Quaternion<T> operator + () const;
			Quaternion<T> operator - () const;
			Quaternion<T> operator + (const Quaternion<T> &qua) const;
			Quaternion<T> operator - (const Quaternion<T> &qua) const;
			Quaternion<T> operator * (const Quaternion<T> &qua) const;
			Quaternion<T> operator * (T t) const;
			Quaternion<T> operator / (T t) const;
			bool operator == (const Quaternion<T> &qua) const;
			bool operator != (const Quaternion<T> &qua) const;
			friend Quaternion<T> operator * (T , const Quaternion<T> &);
			T Length();
			T LengthSquared();
			Vector3<T> xyz();
			Vector4<T> ToVector4();
			void Conjugate();
			void Normalize();
			void Identity();
			Quaternion<T> Zero();
			Quaternion<T> One();
			void ConsoleOut();
			friend std::ostream& operator << (std::ostream &output, const Quaternion<T> &quat)
			{ output << "(x, y, z, w) = (" << quat.x << ", " << quat.y << ", " << quat.z << ", " <<  ")"; return output; }
		};
		#pragma endregion // Quaternion 

		#pragma region Quaterion Functions
		////////////////////////////////////////////////////////////////////////////
		// Quaternion Function
		////////////////////////////////////////////////////////////////////////////

		//-----------------------------------------------------------------------------------------------------
		// Name : Dot()
		// Desc : 2̃NH[^jI̓ςԂ
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate T Dot(Quaternion<T> q1, Quaternion<T> q2);
		AsuraTemplate void Dot(Quaternion<T> &q1, Quaternion<T> &q2, Quaternion<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : Conjugate()
		// Desc : NH[^jI̋Ԃ
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Quaternion<T> Conjugate(Quaternion<T> &qua);
		AsuraTemplate void Conjugate(Quaternion<T> &qua, Quaternion<T> &result);

		//----------------------------------------------------------------------------------------------------
		// Name : Normalize()
		// Desc : 𐳋K̃NH[^jIԂ
		//----------------------------------------------------------------------------------------------------
		AsuraTemplate Quaternion<T> Normalize(Quaternion<T> qua);
		AsuraTemplate void Normalize(Quaternion<T> &qua, Quaternion<T> &result);
			
		//---------------------------------------------------------------------------------------------------
		// Name : Inverse()
		// Desc : NH[^jIčĐK
		//---------------------------------------------------------------------------------------------------
		AsuraTemplate Quaternion<T> Inverse(Quaternion<T> qua);
		AsuraTemplate void Inverse(Quaternion<T> &qua, Quaternion<T> &result);

		//---------------------------------------------------------------------------------------------------
		// Name : CreateFromYawPitchRoll
		// Desc : [Esb`E[w肵ăNH[^jI쐬
		//---------------------------------------------------------------------------------------------------
		AsuraTemplate Quaternion<T> CreateFromYawPitchRoll(T yaw, T pitch, T roll);
		AsuraTemplate void CreateFromYawPitchRoll(T yaw, T pitch, T roll, Quaternion<T> &result);

		//------------------------------------------------------------------------------------------------------
		// Name : Slerp()
		// Desc : ʐ`ԂgăNH[^jIԂԂ
		//------------------------------------------------------------------------------------------------------
		AsuraTemplate Quaternion<T> Slerp(Quaternion<T> q1, Quaternion<T> q2, T t);
		AsuraTemplate void Slerp(Quaternion<T> &q1, Quaternion<T> &q2, T t, Quaternion<T> &result);

		//-----------------------------------------------------------------------------------------------------
		// Name : Squad()
		// Desc : 2ԂgăNH[^jIԂ⊮
		//-----------------------------------------------------------------------------------------------------
		AsuraTemplate Quaternion<T> Squad(
			Quaternion<T> q1, Quaternion<T> a,
			Quaternion<T> b, Quaternion<T> c, T t);
		AsuraTemplate void Squad(
			 Quaternion<T> &q1, Quaternion<T> &a,
			 Quaternion<T> &b, Quaternion<T> &c, T t, Quaternion<T> &result);

		#pragma endregion // Quaternion Functions

		#pragma region Color
		//////////////////////////////////////////////////////////////////////////
		// Color class
		//////////////////////////////////////////////////////////////////////////
		AsuraTemplate
		class Color
		{
		public:
			float r, g, b, a;
			Color<T>() : r(0.0), g(0.0), b(0.0), a(0.0) {}
			Color<T>( ulong argb );
			Color<T>( const T * pf );
			Color<T>( T nr, T ng, T nb, T na ) : r(nr), g(ng), b(nb), a(na) {}
			Color<T>( Vector2<T> &vec, T nb, T na ) : r(vec.x), g(vec.y), b(nb), a(na) {}
			Color<T>( Vector3<T> &vec, T na ) : r(vec.x), g(vec.y), b(vec.z), a(na) {}
			Color<T>( Vector4<T> &vec ) : r(vec.x), g(vec.y), b(vec.z), a(vec.w) {}

			operator ulong () const;

			operator T * ();
			operator const T * () const;

			Color<T>& operator += ( const Color<T>& col );
			Color<T>& operator -= ( const Color<T>& col );
			Color<T>& operator *= ( T f );
			Color<T>& operator /= ( T f );
			Color<T>& operator = ( const Color<T>& col );

			Color<T> operator + () const;
			Color<T> operator - () const;

			Color<T> operator + ( const Color<T>& col ) const;
			Color<T> operator - ( const Color<T>& col ) const;
			Color<T> operator * ( T f ) const;
			Color<T> operator / ( T f ) const;

			friend Color<T> operator * ( T f, const Color<T>& col ) { return Color<T>(f*col.r, f*col.g, f*col.b, f*col.a); }

			bool operator == ( const Color<T>& col ) const;
			bool operator != ( const Color<T>& col ) const;

			void ConsoleOut();
			Color<T> One();
			Color<T> Zero();

			Vector4<T> ToVector4();
			Vector3<T> ToVector3();

			friend std::ostream& operator << (std::ostream &output, const Color<T> &color)
			{ 
				output << "(r, g, b, a) = (" << color.r << ", " << color.g << ", " << color.b << ", " << color.a << ")"; 
				return output;
			}

			Color<T> AliceBlue();
			Color<T> AntiqueWhite();
			Color<T> Aqua();
			Color<T> Aquamarine();
			Color<T> Azure();
			Color<T> Beige();
			Color<T> Bisque();
			Color<T> Black();
			Color<T> BlanchedAlmond();
			Color<T> Blue();
			Color<T> BlueViolet();
			Color<T> Brown();
			Color<T> BurlyWood();
			Color<T> CadeBlue();
			Color<T> Charteuse();
			Color<T> Chocolate();
			Color<T> Coral();
			Color<T> CornflowerBlue();
			Color<T> Cornsilk();
			Color<T> Crimson();
			Color<T> Cyan();
			Color<T> DarkBlue();
			Color<T> DarkCyan();
			Color<T> DarkGoldenrod();
			Color<T> DarkGray();
			Color<T> DarkGreen();
			Color<T> DarkKhaki();
			Color<T> DarkMagenta();
			Color<T> DarkOliveGreen();
			Color<T> DarkOrange();
			Color<T> DarkOrchid();
			Color<T> DarkRed();
			Color<T> DarkSalmon();
			Color<T> DarkSeaGreen();
			Color<T> DarkSlateBlue();
			Color<T> DarkTruquioise();
			Color<T> DarkViolet();
			Color<T> DeepPink();
			Color<T> DeepSkyBlue();
			Color<T> DimGray();
			Color<T> DodgerBlue();
			Color<T> Firebrick();
			Color<T> FloralWhite();
			Color<T> ForestGeen();
			Color<T> Fuchsia();
			Color<T> Gainsboro();
			Color<T> GhostWhite();
			Color<T> Gold();
			Color<T> Goldenrod();
			Color<T> Gray();
			Color<T> Green();
			Color<T> GreenYellow();
			Color<T> Honeydew();
			Color<T> HotPink();
			Color<T> IndianRed();
			Color<T> Indigo();
			Color<T> Ivory();
			Color<T> Khaki();
			Color<T> Lavender();
			Color<T> LavenderBlush();
			Color<T> LawnGreen();
			Color<T> LemonChiffon();
			Color<T> LightBlue();
			Color<T> LightCoral();
			Color<T> LightCyan();
			Color<T> LightGoldenrodYellow();
			Color<T> LightGray();
			Color<T> LightGreen();
			Color<T> LightPink();
			Color<T> LightSalmon();
			Color<T> LightSeaGreen();
			Color<T> LightSkyBlue();
			Color<T> LightSlateGray();
			Color<T> LightSteelBlue();
			Color<T> LightYellow();
			Color<T> Lime();
			Color<T> LimeGreen();
			Color<T> Linen();
			Color<T> Magenta();
			Color<T> Maroon();
			Color<T> MediumAquamarine();
			Color<T> MediumBlue();
			Color<T> MediumOrchid();
			Color<T> MediumPurple();
			Color<T> MediumSeaGreen();
			Color<T> MediumSlateBlue();
			Color<T> MediumSpringGreen();
			Color<T> MediumTruquoise();
			Color<T> MediumVioletRed();
			Color<T> MidnightBlue();
			Color<T> MintCream();
			Color<T> MistyRose();
			Color<T> Moccasin();
			Color<T> NavajoWhite();
			Color<T> Navy();
			Color<T> OldLace();
			Color<T> Olive();
			Color<T> OliveDrad();
			Color<T> Orange();
			Color<T> OrangeRed();
			Color<T> Orchid();
			Color<T> PaleGoldenrod();
			Color<T> PaleGreen();
			Color<T> PaleTurquoise();
			Color<T> PaleVioletRed();
			Color<T> PapayaWhip();
			Color<T> PeachPuff();
			Color<T> Peru();
			Color<T> Pink();
			Color<T> Plum();
			Color<T> PowderBlue();
			Color<T> Purple();
			Color<T> Red();
			Color<T> RosyBrown();
			Color<T> RoyalBlue();
			Color<T> SaddleBrown();
			Color<T> Salmon();
			Color<T> SandyBrown();
			Color<T> SeaGreen();
			Color<T> SeaShell();
			Color<T> Sienna();
			Color<T> Silver();
			Color<T> SkyBlue();
			Color<T> SlateBlue();
			Color<T> SlateGray();
			Color<T> Snow();
			Color<T> SpringGreen();
			Color<T> SteelBlue();
			Color<T> Tan();
			Color<T> Teal();
			Color<T> Thistle();
			Color<T> Tomato();
			Color<T> TransparentBlack();
			Color<T> TransparentWhite();
			Color<T> Turquoise();
			Color<T> Violet();
			Color<T> Wheat();
			Color<T> White();
			Color<T> WhiteSmoke();
			Color<T> Yellow();
			Color<T> YellowGreen();
		};
		#pragma endregion // Color

		#pragma region Color Functions
		///////////////////////////////////////////////////////////////////////
		// Color Functions
		///////////////////////////////////////////////////////////////////////
		AsuraTemplate void Negative( Color<T>&col, Color<T>& result );
		AsuraTemplate Color<T> Negative( Color<T> col);
		AsuraTemplate void Lerp( Color<T>& c1, Color<T>& c2, T f, Color<T>& result );
		AsuraTemplate Color<T> Lerp( Color<T> c1, Color<T> c2, T f );
		ulong ColorARGB( ulong a, ulong r, ulong g, ulong b ); 
		ulong ColorRGBA( ulong r, ulong g, ulong b, ulong a );
		ulong ColorXRGB( ulong r, ulong g, ulong b );
		#pragma endregion // Color Functions

	}	// namespace Math
	#pragma endregion // Math

}	// namespace Asura
#pragma endregion // Asura

#pragma region Inline Files
//
// Inline File
//
#include "AsuraMath.inl"
#pragma endregion // Inline Files

#endif	// _ASURA_MATH_H_INCLUDED_