//------------------------------------------------------------------------
// File : LSPSM.cpp
// Desc : Light Space Perspective Shadow Maps
// Date : May 30, 2010
// Author : Pocol
//------------------------------------------------------------------------

//
// Includes
//
#include "LSPSM.h"

//
// Defines
//
#define NEW_FORMULA 1

//------------------------------------------------------------------------
// Name : TransformCoord()
// Desc : s3xNgϊ
//------------------------------------------------------------------------
VECTOR3 TransformCoord(VECTOR3 point, MATRIX matrix)
{
	float x = matrix._11 * point.x + matrix._21 * point.y + matrix._31 * point.z + matrix._41;
	float y = matrix._12 * point.x + matrix._22 * point.y + matrix._32 * point.z + matrix._42;
	float z = matrix._13 * point.x + matrix._23 * point.y + matrix._33 * point.z + matrix._43;
	float w = matrix._14 * point.x + matrix._24 * point.y + matrix._34 * point.z + matrix._44;

	x /= w;
	y /= w;
	z /= w;
	return VECTOR3( x, y, z );
}

//////////////////////////////////////////////////////////////////////////
// PointList Class
//////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------
// Name : PointList()
// Desc : RXgN^
//------------------------------------------------------------------------
PointList::PointList()
{
	//XgNA
	m_list.clear();
}

//------------------------------------------------------------------------
// Name : GetSize()
// Desc : TCYԂ
//------------------------------------------------------------------------
int PointList::GetSize()
{
	return (int)m_list.size();
}

//------------------------------------------------------------------------
// Name : Add()
// Desc : Wǉ
//------------------------------------------------------------------------
void PointList::Add(VECTOR3& value)
{
	m_list.push_back( value );
}

//------------------------------------------------------------------------
// Name : Clear()
// Desc : XgNA
//------------------------------------------------------------------------
void PointList::Clear()
{
	m_list.clear();
}

//------------------------------------------------------------------------
// Name : Transform()
// Desc : w肵sŃXgϊ
//------------------------------------------------------------------------
void PointList::Transform(MATRIX matrix)
{
	int size = (int)m_list.size();
	for( int i=0; i<size; i++ )
	{
		VECTOR3 point = m_list[i];
		m_list[i] = TransformCoord( point, matrix );
	}
}

//------------------------------------------------------------------------
// Name : ComputeBoundingBox()
// Desc : AABBZo
//------------------------------------------------------------------------
void PointList::ComputeBoundingBox(VECTOR3& min, VECTOR3& max)
{
	int size = (int)m_list.size();
	if( size > 0 )
	{
		min = max = m_list[0];
		for( int i=1; i<size; i++ )
		{
			min = Min( min, m_list[i] );
			max = Max( max, m_list[i] );
		}
	}
}

//------------------------------------------------------------------------
// operator =
//------------------------------------------------------------------------
PointList& PointList::operator = (const PointList &value)
{
	m_list.clear();
	int size = (int)value.m_list.size();
	for( int i=0; i<size; i++ )
	{
		m_list.push_back( value.m_list[i] );
	}
	return (*this);
}

//------------------------------------------------------------------------
// operator []
//------------------------------------------------------------------------
VECTOR3& PointList::operator[](int index)
{
	return m_list[index];
}


//////////////////////////////////////////////////////////////////////////
// LSPSMCalculator Class
//////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------
// Name : LSPSMCalculator()
// Desc : RXgN^
//------------------------------------------------------------------------
LSPSMCalculator::LSPSMCalculator()
{
	nearClip = 0.1f;
	useLSPSM = true;
	pointList.Clear();
}

//------------------------------------------------------------------------
// Name : ComputeUpVector()
// Desc : _ECgxNgAbvxNgvZ
//-------------------------------------------------------------------------
VECTOR3 LSPSMCalculator::ComputeUpVector(VECTOR3 viewDirection, VECTOR3 lightDirection)
{
	VECTOR3 left;
	VECTOR3 up;

	left = Cross( lightDirection, viewDirection );
	up = Cross( left, lightDirection );
	up.Normalize();
	return up;
}

//------------------------------------------------------------------------
// Name : ComputeMatrix_USM()
// Desc : ʏ̃VhE}bvsvZ
//------------------------------------------------------------------------
void LSPSMCalculator::ComputeMatrix_USM()
{
	VECTOR3 max, min;

	//Cg̃r[s߂
	lightView = CreateLookAt( eyePosition, (eyePosition + lightDirection), viewDirection );

	//CgԂ֕ϊ
	pointList.Transform( lightView );

	//AABBZo
	pointList.ComputeBoundingBox( min, max );

	//͈͂Kɂ
	lightProjection = GetUnitCubeClipMatrix( min, max );
}

//------------------------------------------------------------------------
// Name : ComputeMatrix_LSPSM()
// Desc : CgԓVhE}bvsvZ
//------------------------------------------------------------------------
void LSPSMCalculator::ComputeMatrix_LSPSM()
{
	VECTOR3 max, min;

	//xNgƃCgxNĝȂpx߂
	float angle = GetCrossingAngle(viewDirection, lightDirection);

	//Ȃp0x܂180x̏ꍇ
	if ( angle == 0.0f || angle == Pif )
	{
		//Cgɂc݂Ȃ̂Œʏ̃VhE}bvKp
		ComputeMatrix_USM();
		return;
	}

	//XgRs[Ă
	PointList listClone = pointList;

	float sinGamma = sqrtf( 1.0f - angle * angle );

	// AbvxNgZo
	VECTOR3 up = ComputeUpVector( viewDirection, lightDirection );

	//Cg̃r[s߂
	lightView = CreateLookAt( eyePosition, (eyePosition + lightDirection), up );

	//Cg̃r[sŃXgϊAAABBZo
	pointList.Transform( lightView );
	pointList.ComputeBoundingBox( min, max );

	//V߂
	const float factor = 1.0f / sinGamma;
	const float z_n = factor * nearClip;
	const float d = abs( max.y - min.y );
#if NEW_FORMULA 
	// New Formula written in ShaderX4
	const float z0 = - z_n;
	const float z1 = - ( z_n + d * sinGamma );
	const float n = d / ( sqrtf( z1 / z0 ) - 1.0f );
#else
 	// Old Formula written in papers
	const float z_f = z_n + d * sinGamma;
	const float n = ( z_n + sqrtf( z_f * z_n ) ) * factor;
#endif
	const float f = n + d;
	VECTOR3 pos = eyePosition - up * ( n - nearClip );

	//VhE}bvpCg̃r[s
	lightView = CreateLookAt( pos, (pos + lightDirection), up );

	//Yւ̎ˉes擾
	MATRIX proj = GetPerspective( n, f );

	//ϊ̋Ԃ֕ϊ
	Multiply( lightView, proj, lightProjection );
	listClone.Transform( lightProjection );

	//AABBZo
	listClone.ComputeBoundingBox( min, max );

	//͈͂Kɂ
	MATRIX clip = GetUnitCubeClipMatrix( min, max );

	//VhE}bvpCg̎ˉes߂
	Multiply( proj, clip, lightProjection );
}

//------------------------------------------------------------------------
// Name : UpdateShadowMatrix()
// Desc : Cg̍sXV
//------------------------------------------------------------------------
void LSPSMCalculator::UpdateShadowMatrix()
{
	MATRIX viewProj;
	Multiply( eyeView, eyeProjection, viewProj );

	//r[{[߂
	ComputeLightVolumePoints( viewProj );

	// Light Space Perspective Shadow Map
	ComputeMatrix_LSPSM();

	//Wnɕϊ
	MATRIX scale = CreateScale( 1.0f, 1.0f, -1.0f );
	Multiply( lightProjection, scale, lightProjection );

	//VhE}bvɎgŏIIȍs߂
	Multiply( lightView, lightProjection, lightViewProjection );
}

//-----------------------------------------------------------------------
// Name : ComputeLightVolumePoints()
// Desc : ʑ̂߂
//-----------------------------------------------------------------------
void LSPSMCalculator::ComputeLightVolumePoints(MATRIX viewProj)
{
	//J̎߂
	ComputeViewFrustum( viewProj );

	//J̎ɉe𓊂IuWFNg邩肷
	// Check 1: ƌĂ邩H
	/*** `镽ʂAABBƂ̌s ***/
	///ʓ|Ȃ̂Ŗ///

	// Check 2: IuWFNg̊e_烉CgxNg
	// C΂邩H
	/*** e_烉CgxNgɃC΂A`镽ʂƂ̌s ***/
	///ʓ|Ȃ̂Ŗ///
}

//-----------------------------------------------------------------------
// Name : ComputeViewFrustum()
// Desc : J̎߂
//-----------------------------------------------------------------------
void LSPSMCalculator::ComputeViewFrustum(MATRIX viewProj )
{
	pointList.Clear();

	//@̂쐬
	VECTOR3 v[8];
	v[0] = VECTOR3( -1.0f, +1.0f, -1.0f );
	v[1] = VECTOR3( -1.0f, -1.0f, -1.0f );
	v[2] = VECTOR3( +1.0f, -1.0f, -1.0f );
	v[3] = VECTOR3( +1.0f, +1.0f, -1.0f );
	v[4] = VECTOR3( -1.0f, +1.0f, +1.0f );
	v[5] = VECTOR3( -1.0f, -1.0f, +1.0f );
	v[6] = VECTOR3( +1.0f, -1.0f, +1.0f );
	v[7] = VECTOR3( +1.0f, +1.0f, +1.0f );
	for( int i=0; i<8; i++ )
	{
		pointList.Add( v[i] );
	}

	//@r[s񁨎ˉes̋tϊss߂
	MATRIX invViewProj = Invert( viewProj );

	//@̂ɋtϊsA߂
	pointList.Transform( invViewProj );
}

//-----------------------------------------------------------------------
// Name : GetUnitCubeClipMatrix()
// Desc : ͈͂Kɂs擾
//-----------------------------------------------------------------------
MATRIX LSPSMCalculator::GetUnitCubeClipMatrix(VECTOR3 min, VECTOR3 max)
{
	MATRIX result;

	result._11 = 2.0f / ( max.x - min.x );
	result._21 = 0.0f;
	result._31 = 0.0f;
	result._41 = -( max.x + min.x ) / ( max.x - min.x );

	result._12 = 0.0f;
	result._22 = 2.0f / ( max.y - min.y );
	result._32 = 0.0f;
	result._42 = -( max.y + min.y ) / ( max.y - min.y );

	result._13 = 0.0f;
	result._23 = 0.0f;
	result._33 = 1.0f / ( max.z - min.z );
	result._43 = - min.z / ( max.z - min.z );

	result._14 = 0.0f;
	result._24 = 0.0f;
	result._34 = 0.0f;
	result._44 = 1.0f;

	return result;
}

//-----------------------------------------------------------------------
// Name : GetPerspective()
// Desc : Yւ̎ˉes擾
//-----------------------------------------------------------------------
MATRIX LSPSMCalculator::GetPerspective( float nearDist, float farDist )
{
	MATRIX result;
	result.Identity();
	result._22 = farDist /( farDist - nearDist );
	result._24 = 1.0f;
	result._42 = -farDist * nearDist /( farDist - nearDist );
	result._44 = 0.0f;
	return result;
}

//-----------------------------------------------------------------------
// Name : SetEyePos()
// Desc : J̈ʒuݒ
//-----------------------------------------------------------------------
void LSPSMCalculator::SetEyePos( VECTOR3 value )
{
	eyePosition = value;
}

//-----------------------------------------------------------------------
// Name : SetViewDir()
// Desc : J̎xNgݒ
//-----------------------------------------------------------------------
void LSPSMCalculator::SetViewDir( VECTOR3 value )
{
	viewDirection = value;
}

//-----------------------------------------------------------------------
// Name : SetLightDir()
// Desc : CgxNgݒ
//-----------------------------------------------------------------------
void LSPSMCalculator::SetLightDir( VECTOR3 value )
{
	lightDirection = value;
}

//-----------------------------------------------------------------------
// Name : SetEyeView()
// Desc : J̃r[sݒ
//-----------------------------------------------------------------------
void LSPSMCalculator::SetEyeView( MATRIX value )
{
	eyeView = value; 
}

//-----------------------------------------------------------------------
// Name : SetEyeProjection()
// Desc : J̎ˉesݒ
//-----------------------------------------------------------------------
void LSPSMCalculator::SetEyeProjection( MATRIX value )
{
	eyeProjection = value;
}

//-----------------------------------------------------------------------
// Name : SetNearClip()
// Desc : jAʂ܂ł̋ݒ肷
//-----------------------------------------------------------------------
void LSPSMCalculator::SetNearClip( float value )
{
	nearClip = value;
}

//-----------------------------------------------------------------------
// Name : SetUseLSPSM()
// Desc : tO̐ݒ
//-----------------------------------------------------------------------
void LSPSMCalculator::SetUseLSPSM( bool value )
{
	useLSPSM = value;
}

//-----------------------------------------------------------------------
// Name : GetLightView()
// Desc : Cg̃r[s擾
//-----------------------------------------------------------------------
MATRIX LSPSMCalculator::GetLightView()
{
	return lightView; 
}

//-----------------------------------------------------------------------
// Name : GetLightProjection()
// Desc : Cg̎ˉes擾
//-----------------------------------------------------------------------
MATRIX LSPSMCalculator::GetLightProjection()
{
	return lightProjection;
}

//-----------------------------------------------------------------------
// Name : GetLightViewProjection()
// Desc : Cg̃r[ˉes擾
//-----------------------------------------------------------------------
MATRIX LSPSMCalculator::GetLightViewProjection()
{
	return lightViewProjection;
}

