//-----------------------------------------------------------------------
// File : LSPSM.h
// Desc : Light Space Perspective Shadow Maps
// Date : May 30, 2010
// Author : Pocol
//-----------------------------------------------------------------------

#ifndef __LSPSM_H__
#define __LSPSM_H__

//
// Includes
//
#include <iostream>
#include <vector>
#include "AsuraFrameWork/AsuraMath.h"

//
// Using Statements
//
using namespace Asura::Math;

//
// Typedef
//
typedef Vector3f VECTOR3;
typedef Vector4f VECTOR4;
typedef Matrix4f MATRIX;

/////////////////////////////////////////////////////////////////////////
// PointList Class
/////////////////////////////////////////////////////////////////////////
class PointList
{
private:
	std::vector<VECTOR3> m_list;

public:
	PointList();
	int GetSize();
	void Add( VECTOR3& value );
	void Clear();
	void Transform( MATRIX matrix );
	void ComputeBoundingBox( VECTOR3& min, VECTOR3& max );
	
	PointList& operator= ( const PointList& value );
	VECTOR3& operator[] ( int index );
};

/////////////////////////////////////////////////////////////////////////
// LSPSMCalculator Class
/////////////////////////////////////////////////////////////////////////
class LSPSMCalculator
{
private:
	VECTOR3 eyePosition;
	VECTOR3 viewDirection;
	VECTOR3 lightDirection;

	MATRIX eyeView;
	MATRIX eyeProjection;
	MATRIX lightView;
	MATRIX lightProjection;
	MATRIX lightViewProjection;

	PointList pointList;
	float nearClip;
	bool useLSPSM;

	void ComputeMatrix_USM();
	void ComputeMatrix_LSPSM();
	void ComputeLightVolumePoints( MATRIX viewProj );
	void ComputeViewFrustum( MATRIX viewProj );
	VECTOR3 ComputeUpVector( VECTOR3 viewDir, VECTOR3 lightDir );
	MATRIX GetUnitCubeClipMatrix( VECTOR3 min, VECTOR3 max );
	MATRIX GetPerspective( float nearDist, float farDist );

public:
	LSPSMCalculator();
	void SetEyePos( VECTOR3 value );
	void SetViewDir( VECTOR3 value );
	void SetLightDir( VECTOR3 value );
	void SetEyeView( MATRIX value );
	void SetEyeProjection( MATRIX value );
	void SetNearClip( float value );
	void UpdateShadowMatrix();
	void SetUseLSPSM( bool value );

	MATRIX GetLightView();
	MATRIX GetLightProjection();
	MATRIX GetLightViewProjection();
};

#endif // __LSPSM_H__