#pragma region File Description
//--------------------------------------------------------------------------------------------------
// File : Main.h
// Desc : 
// Date : Dec. 01, 2008
// Version : 1.0
// Author : Pocol
//--------------------------------------------------------------------------------------------------
#pragma endregion

#ifndef __MAIN_H_INCLUDED__
#define __MAIN_H_INCLUDED__

#pragma region Disable Warning C4996
//
// Disable Warning C4996
//
#ifndef _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#endif
#ifndef _CRT_SECURE_CPP_OVERLOAD_SECURE_NAMES
#define _CRT_SECURE_CPP_OVERLOAD_SECURE_NAMES 1
#endif
#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS 1
#endif
#pragma endregion

#pragma region Includes
//
// Includes
//
#include <iostream>
#include <GL/glew.h>
#include <GL/glut.h>
#include "AsuraFrameWork/AsuraMath.h"
#include "AsuraFrameWork/AsuraTexture.h"
#include "AsuraFrameWork/AsuraMesh.h"
#pragma endregion

#pragma region Using Statements
//
// Using Statements
//
using namespace std;
using namespace Asura;
using namespace Asura::Math;
#pragma endregion

#pragma region Link
//
// Link
//
#pragma comment( lib, "glew32.lib" )
#pragma endregion

#pragma region Defines
//
// Defines
//
#if defined(DEBUG) || defined(_DEBUG)
#define DEFINE_DEBUG 1
#define DEFINE_RELEASE 0
#else
#define DEFINE_DEBUG 0
#define DEFINE_RELEASE 1
#endif
#ifndef SAFE_DELETE
#define SAFE_DELETE(x) { if (x) { delete (x); (x) = NULL; } }
#endif
#ifndef SAFE_DELETE_ARRAY
#define SAFE_DELETE_ARRAY(x) { if (x) { delete [] (x); (x) = NULL; } }
#endif
#pragma endregion

#pragma region WindowData Struct
////////////////////////////////////////////////////////////////////////
// WindowData
////////////////////////////////////////////////////////////////////////
struct WindowData
{
	int Width;
	int Height;
	float AspectRatio;
	bool FullScreen;
	char* Title;
	char* Version;
	char* Vendor;
	char* Renderer;
	char FramePerSecond[15];
	float FrameRate;
	double CurrentTime;
	double PreviousTime;
	int CurrentCount;
	int PreviousCount;
	int StartCount;
	int Frame;

	WindowData( int width, int height, char* title )
	{
		Width = width;
		Height = height;
		Title = new char [strlen(title)+1];
		strcpy( Title, title );
		CurrentTime = 0;
		PreviousTime = 0;
		CurrentCount = 0;
		PreviousCount = 0;
		FrameRate = 0;
		Frame = 0;
	}

	void Initialize()
	{
		const GLubyte *version = glGetString( GL_VERSION );
		const GLubyte *vendor = glGetString( GL_VENDOR );
		const GLubyte *renderer = glGetString( GL_RENDERER );
		Version = new char [ 7 + strlen((char*)version) + 1 ];
		Vendor = new char [ strlen((char*)vendor) + 1 ];
		Renderer = new char [ strlen((char*)renderer) + 1 ];
		sprintf( Version, "OpenGL %s", version );
		sprintf( Vendor, "%s", vendor );
		sprintf( Renderer, "%s", renderer );
		CurrentCount = PreviousCount = StartCount = glutGet( GLUT_ELAPSED_TIME );
	}

	void Release()
	{
		SAFE_DELETE_ARRAY( Title );
		SAFE_DELETE_ARRAY( Version );
		SAFE_DELETE_ARRAY( Vendor );
		SAFE_DELETE_ARRAY( Renderer );
	}

	float GetFPS()
	{ 
		CurrentCount = glutGet( GLUT_ELAPSED_TIME );
		CurrentTime = ( CurrentCount - PreviousCount ) * 0.001;
		Frame++;
		double delta = ( CurrentTime - PreviousTime );
		if ( delta >= 1.0 )
		{
			FrameRate = (float)Frame/(float)delta;
			Frame = 0;
			PreviousTime = CurrentTime;
			sprintf( FramePerSecond, "FPS : %3.2f", FrameRate );
		}
		return FrameRate;
	}
};
#pragma endregion

#pragma region Functions
////////////////////////////////////////////////////////////////////////
// Functions
////////////////////////////////////////////////////////////////////////
bool Initialize();
void Shutdown();
void ReshapeFunction( int x, int y);
void RenderFunction();
void IdleFunction();
void KeyboardFunction( unsigned char key, int x, int y );
void SpecialFunction( int key, int x, int y );
void MouseFunction( int button, int state, int x, int y );
void MotionFunction( int x, int y );
void PassiveMotionFunction( int x, int y );
void MenuFunction( int item );
void Draw2D( double time, double delta, float fps );
void Draw3D( double time, double delta, float fps );

void glutRenderText( void* font, char *text )
{ for ( int i=0; i<(int)strlen(text); i++ ) glutBitmapCharacter( font, (int)text[i] ); }
void glSafeDeleteLists( GLuint list, GLsizei size )
{ if ( list ) { glDeleteLists( list, size ); } }

#pragma endregion

#endif // __MAIN_H_INCLUDED__