#pragma region File Description
//-------------------------------------------------------------------------------------------------
// File : PerspectiveShadowMap.cpp
// Desc : Perspective Shadow Map
// Date : Nov. 29, 2008
// Version : 1.0
// Author : Pocol
//-------------------------------------------------------------------------------------------------
#pragma endregion

#pragma region Includes
//
// Includes
//
#include <GL/glew.h>
#include "PSM.h"
#include <GL/glut.h>
#pragma endregion

#pragma region PSMCalculator Class
////////////////////////////////////////////////////////////////////////
// PSMCalculator class
////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------------------------------
// Name : PSMCalculator()
// Desc : RXgN^
//--------------------------------------------------------------------------------------------------
PSMCalculator::PSMCalculator()
{
	//@l
	Fov = DEFAULT_FIELD_OF_VIEW;
	AspectRatio = DEFAULT_ASPECT_RATIO;
	NearClip = DEFAULT_NEAR_CLIP;
	FarClip = DEFAULT_FAR_CLIP;
	CameraPosition = Vector3f( 3.0f, 3.0f, -3.0f );
	WorldSpaceLight = Vector4f( 1.0f, 2.0f, 0.0f, 0.0f );
	WorldSpaceLight.Normalize();
	PostPerspectiveSpaceLight = Vector4f( 0.0f, 0.0f, 0.0f, 0.0f );
	View = Matrix4f();
	Projection = Matrix4f();
	ViewProjection = Matrix4f();
	LightView = Matrix4f();
	LightProjection = Matrix4f();
	LightViewProjection = Matrix4f();
}

//--------------------------------------------------------------------------------------------------
// Name : ~PSMCalculator()
// Desc : fXgN^
//--------------------------------------------------------------------------------------------------
PSMCalculator::~PSMCalculator()
{
}

//--------------------------------------------------------------------------------------------------
// Name : SetFieldOfView()
// Desc : p̐ݒ
//--------------------------------------------------------------------------------------------------
void PSMCalculator::SetFieldOfView( float fov )
{ Fov = fov; }

//--------------------------------------------------------------------------------------------------
// Name : SetAspectRatio()
// Desc : AXyNg̐ݒ
//--------------------------------------------------------------------------------------------------
void PSMCalculator::SetAspectRatio( float aspectRatio )
{ AspectRatio = aspectRatio; }

//--------------------------------------------------------------------------------------------------
// Name : SetNearClip()
// Desc : ߕʂ܂ł̋ݒ
//--------------------------------------------------------------------------------------------------
void PSMCalculator::SetNearClip( float nearclip )
{ NearClip = nearclip; }

//--------------------------------------------------------------------------------------------------
// Name : SetFarClip()
// Desc : ʂ܂ł̋ݒ
//--------------------------------------------------------------------------------------------------
void PSMCalculator::SetFarClip( float farclip )
{ FarClip = farclip; }

//--------------------------------------------------------------------------------------------------
// Name : SetProjectionParameter()
// Desc : ˉes쐬p̃p[^ݒ肷
//--------------------------------------------------------------------------------------------------
void PSMCalculator::SetProjectionParameter( float fov, float aspectRatio, float nearclip, float farclip )
{
	Fov = fov;
	AspectRatio = aspectRatio;
	NearClip = nearclip;
	FarClip = farclip;
}

//--------------------------------------------------------------------------------------------------
// Name : SetCameraPosition()
// Desc : Jʒuݒ肷
//--------------------------------------------------------------------------------------------------
void PSMCalculator::SetCameraPosition( Vector3f position )
{ CameraPosition = position; }

//--------------------------------------------------------------------------------------------------
// Name : SetWorldSpaceLight()
// Desc : [hԂł̃Cgݒ肷
//--------------------------------------------------------------------------------------------------
void PSMCalculator::SetWorldSpaceLight(Vector3f direction)
{ WorldSpaceLight = Vector4f( direction, 0.0f ); }

//--------------------------------------------------------------------------------------------------
// Name : GetView()
// Desc : r[s擾
//--------------------------------------------------------------------------------------------------
Matrix4f PSMCalculator::GetView()
{ return View; }

//--------------------------------------------------------------------------------------------------
// Name : GetProjection()
// Desc : ˉes擾
//--------------------------------------------------------------------------------------------------
Matrix4f PSMCalculator::GetProjection()
{ return Projection; }

//--------------------------------------------------------------------------------------------------
// Name : GetViewProjection()
// Desc : r[ˉes擾
//--------------------------------------------------------------------------------------------------
Matrix4f PSMCalculator::GetViewProjection()
{ return ViewProjection; }

//--------------------------------------------------------------------------------------------------
// Name : GetLightView()
// Desc : Cg̃r[s擾
//--------------------------------------------------------------------------------------------------
Matrix4f PSMCalculator::GetLightView()
{ return LightView; }

//--------------------------------------------------------------------------------------------------
// Name : GetLightProjection()
// Desc : Cg̎ˉes擾
//--------------------------------------------------------------------------------------------------
Matrix4f PSMCalculator::GetLightProjection()
{ return LightProjection; }

//--------------------------------------------------------------------------------------------------
// Name : GetLightViewProjection()
// Desc : Cg̃r[ˉes擾
//--------------------------------------------------------------------------------------------------
Matrix4f PSMCalculator::GetLightViewProjeciton()
{ return LightViewProjection; }

//--------------------------------------------------------------------------------------------------
// Name : GetWorldSpaceLight()
// Desc : [hԂ̕Cg擾
//--------------------------------------------------------------------------------------------------
Vector4f PSMCalculator::GetWorldSpaceLight()
{ return WorldSpaceLight; }

//--------------------------------------------------------------------------------------------------
// Name : GetPostPerspectiveSpaceLight()
// Desc : ϊ̋Ԃ̃Cg擾
//--------------------------------------------------------------------------------------------------
Vector4f PSMCalculator::GetPostPerspectiveSpaceLight()
{ return PostPerspectiveSpaceLight; }

//--------------------------------------------------------------------------------------------------
// Name : GetFinalMatrix()
// Desc : PSMɗpŏIIȍs擾
//--------------------------------------------------------------------------------------------------
Matrix4f PSMCalculator::GetFinalMatrix()
{ return FinalMatrix; }

//--------------------------------------------------------------------------------------------------
// Name : ComputeMatrix()
// Desc : PSMp̍svZ
//--------------------------------------------------------------------------------------------------
Matrix4f PSMCalculator::ComputeMatrix()
{
	//@ˉeEr[s쐬
#if NO_USE_GLUPERSPECTIVE
	Projection = CreatePerspectiveFieldOfView( Fov, AspectRatio, NearClip, FarClip );
#else
	Projection = CreateGLUPerspective( Fov, AspectRatio, NearClip, FarClip );
#endif
	View = CreateLookAt( CameraPosition, Vector3f().Zero(), Vector3f().Up() );

	Vector4f wsLight = WorldSpaceLight;
	wsLight.Normalize();	//@K

	//@f1Fߕʂ̍ŏ
	float d = 0.0f;
	if ( NearClip < MIN_Z_NEAR )
	{
		//@߂ߕʂ邽߂̃ItZbg
		d = MIN_Z_NEAR - NearClip;
	}

	//@f2Fϑ҂̌납e𗎂ƂIuWFNg邩H

	//@xNg
	Vector4f lookAtDir = Vector4f( 
		View.m[0][2], 
		View.m[1][2],
		View.m[2][2],
		0.0f );

	//@ȂpZo
	float dot = Dot( wsLight, lookAtDir );

	//@J̌납烉Cgꍇ
	if ( dot < 0.0f && fabs( wsLight.y ) > EPSILON )
	{
		float t = -( MAX_Y_SCENE - CameraPosition.y )/ wsLight.y;

		//@_̔wɂp̃ItZbg
		d = max<float>( d, t * sqrtf( wsLight.x * wsLight.x + wsLight.z * wsLight.z ) );
	}

	//@쐍̒߂KvǂH
	if ( d > 0.0f )
	{
		//@zɂăXChobN
		View.m[3][2] -= d;

		//@쐍̋ߕʂƉʂ𒲐߂āCˉes쐬Ȃ
#if NO_USE_GLUPERSPECTIVE
		Projection = CreatePerspectiveFieldOfView( Fov, AspectRatio, NearClip + d, FarClip + d );
#else
		Projection = CreateGLUPerspective( Fov, AspectRatio, NearClip + d, FarClip + d );
#endif
	}
	
	//@r[ˉesZo
	ViewProjection = View * Projection;

	//@[hԂ̃Cg𓧎ϊ̋Ԃɕϊ
	PostPerspectiveSpaceLight = Transform( wsLight, ViewProjection );

	//@w0̏ꍇ
	if ( fabsf( PostPerspectiveSpaceLight.w ) < EPSILON )
	{
		//@ϊ̃Cg͕s

		//@Cg̍s쐬
		LightProjection = CreateOrthographicOffcenter( -SQRT2, SQRT2, -SQRT2, SQRT2, -SQRT2, SQRT2 );
		LightView = CreateLookAt( Vector3f().Zero(), -PostPerspectiveSpaceLight.xyz(), Vector3f().UnitZ() );

		//@Cg̃r[ˉes
		LightViewProjection = LightView * LightProjection;
	}
	else
	{
		//@ϊ̃Cg͓_
			
		//@ubNz[H
		if ( PostPerspectiveSpaceLight.w < 0.0f )
		{
			//@}CiXĔ]
			Matrix4f Scale = CreateScale( 1.0f, 1.0f, -1.0f );
			PostPerspectiveSpaceLight = Transform( PostPerspectiveSpaceLight, Scale );
		}

		PostPerspectiveSpaceLight *= ( 1.0f/ PostPerspectiveSpaceLight.w );
		PostPerspectiveSpaceLight.w = 0.0f;

		//@NbvԁiPʃL[uj̃oEfBOXtBA܂ނ悤
		//@Cg̎쐍𒲐߂
		float radius = SQRT3;
		float dist = sqrtf( Dot( PostPerspectiveSpaceLight, PostPerspectiveSpaceLight ) );
		float fov = 2.0f * atanf( radius / dist );
		float fNear = max<float>( dist - radius, 0.001f );
		float fFar = dist + radius;

		//@Cg̍s쐬
#if NO_USE_GLUPERSPECTIVE
		LightProjection = CreatePerspectiveFieldOfView( fov, 1.0f, fNear, fFar );
#else
		LightProjection = CreateGLUPerspective( fov, 1.0f, fNear, fFar );
#endif
		LightView = CreateLookAt( PostPerspectiveSpaceLight.xyz(), Vector3f().Zero(), Vector3f().UnitZ() );
	
		//@Cg̃r[ˉes
		LightViewProjection = LightView * LightProjection;
	}

	//@ŏIIȃVhE}bvs
	FinalMatrix = ViewProjection * LightViewProjection;

	//@vZʂԂ
	return FinalMatrix;
}
#pragma endregion

#pragma region Functions
/////////////////////////////////////////////////////////////////////////
// Functions
/////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------------
// Name : CreateGLUPerspective()
// Desc : ˉes쐬
//---------------------------------------------------------------------------------------------------
Matrix4f CreateGLUPerspective(float fov_Rad, float aspectRatio, float nearClip, float farClip)
{
	float yScale = cot( fov_Rad/2.0f );
	float xScale = yScale/aspectRatio;

	Matrix4f result = Matrix4f().Zero();
	result._11 = xScale;
	result._22 = yScale;
	result._33 = (farClip + nearClip)/(nearClip - farClip);
	result._34 = 1.0f;
	result._43 = (2.0f*farClip*nearClip)/(nearClip - farClip);

	return result;
}

//---------------------------------------------------------------------------------------------------
// Name : EnableShadowTexCoord()
// Desc : eNX`W̎LCw肳ꂽ
//@@ seNX`sɏZ
//---------------------------------------------------------------------------------------------------
void EnableShadowTexCoord( Matrix4f value )
{
	const float genS[4] = { 1.0f, 0.0f, 0.0f, 0.0f };
	const float genT[4] = { 0.0f, 1.0f, 0.0f, 0.0f };
	const float genR[4] = { 0.0f, 0.0f, 1.0f, 0.0f };
	const float genQ[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
	const float white[4] = { 1.0f, 1.0f, 1.0f, 1.0f };

	//@eNX`}bvL
	glEnable( GL_TEXTURE_2D );

	//@eNX`W̎L
	glEnable( GL_TEXTURE_GEN_S );
	glEnable( GL_TEXTURE_GEN_T );
	glEnable( GL_TEXTURE_GEN_R );
	glEnable( GL_TEXTURE_GEN_Q );

	//@_Wnɂ镨̂̍Wlp
	glTexGeni( GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );
	glTexGeni( GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );
	glTexGeni( GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );
	glTexGeni( GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR );

	//@eNX`Wg
	glTexGenfv( GL_S, GL_EYE_PLANE, genS );
	glTexGenfv( GL_T, GL_EYE_PLANE, genT );
	glTexGenfv( GL_R, GL_EYE_PLANE, genR );
	glTexGenfv( GL_Q, GL_EYE_PLANE, genQ );

	glTexParameterfv( GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, white );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );

	//@eNX`s
	glMatrixMode( GL_TEXTURE );
	glLoadIdentity();

	//@-1`10`1͈̔͂ɂ
	glTranslatef( 0.5f, 0.5f, 0.5f );
	glScalef( 0.5f, 0.5f, 0.5f );

	//@s
	glMultMatrixf( value );

	//@fr[s
	glMatrixMode( GL_MODELVIEW );
}

//--------------------------------------------------------------------------------------------------
// Name : DisableShadowTexCoord()
// Desc : eNX`W̎𖳌
//--------------------------------------------------------------------------------------------------
void DisableShadowTexCoord()
{
	//@eNX`s
	glMatrixMode( GL_TEXTURE );
	glLoadIdentity();

	//@eNX`̎𖳌
	glDisable( GL_TEXTURE_GEN_S );
	glDisable( GL_TEXTURE_GEN_T );
	glDisable( GL_TEXTURE_GEN_R );
	glDisable( GL_TEXTURE_GEN_Q );
	
	//@fr[s
	glMatrixMode( GL_MODELVIEW );
}
#pragma endregion 

