#pragma region File Description
//--------------------------------------------------------------------------------------------------
// File : PerspetiveShadowMap.h
// Desc : Perspective Shadow Map
// Date : Nov. 29, 2008
// Version : 1.0
// Author : Pocol
//--------------------------------------------------------------------------------------------------
#pragma endregion

#ifndef __PSM_H_INCLUDED__
#define __PSM_H_INCLUDED__

#pragma region Includes
//
// Includes
//
#include "AsuraFrameWork/AsuraMath.h"
#include <GL/glut.h>
#pragma endregion

#pragma region Using Statements
//
// Using Statements
//
using namespace Asura;
using namespace Asura::Math;
#pragma endregion

#pragma region Defines
//
// Defines
//
#define MAX_Y_SCENE ((float)1.1f)
#define MIN_Z_NEAR ((float)4.0f)
#define EPSILON ((float)0.00000011920928955078125f)	// floatľEx(1/2^23)
#define SQRT2 ((float)1.4142135623730950488016887242097f)
#define SQRT3 ((float)1.7320508075688772935274463415059f)
#define DEFAULT_FIELD_OF_VIEW ((float)0.78539816339744830961566084581988f)
#define DEFAULT_ASPECT_RATIO ((float)1.0f)
#define DEFAULT_NEAR_CLIP ((float)3.0f)
#define DEFAULT_FAR_CLIP ((float)20.0f)
#pragma endregion

#pragma region PSMCalculator Class
////////////////////////////////////////////////////////////////////////
// PSMCalculator class
////////////////////////////////////////////////////////////////////////
class PSMCalculator
{
protected:
	float Fov;
	float AspectRatio;
	float NearClip;
	float FarClip;
	Matrix4f View;
	Matrix4f Projection;
	Matrix4f ViewProjection;
	Matrix4f LightView;
	Matrix4f LightProjection;
	Matrix4f LightViewProjection;
	Matrix4f FinalMatrix;
	Vector3f CameraPosition;
	Vector4f WorldSpaceLight;
	Vector4f PostPerspectiveSpaceLight;

	
public:
	PSMCalculator();
	~PSMCalculator();

	void SetProjectionParameter( 
		float fov = (float)PiOver4,
		float aspectRatio=1.0f,
		float nearclip=3.0f,
		float farclip=20.0f );
	void SetCameraPosition( Vector3f position );
	void SetWorldSpaceLight( Vector3f direction );
	void SetAspectRatio( float aspectRatio );
	void SetFieldOfView( float fov );
	void SetFarClip( float farclip );
	void SetNearClip( float nearclip );
	Matrix4f ComputeMatrix();

	Matrix4f GetView();
	Matrix4f GetProjection();
	Matrix4f GetViewProjection();
	Matrix4f GetLightView();
	Matrix4f GetLightProjection();
	Matrix4f GetLightViewProjeciton();
	Matrix4f GetFinalMatrix();
	Vector4f GetWorldSpaceLight();
	Vector4f GetPostPerspectiveSpaceLight();
};
#pragma endregion

#pragma region Functions
////////////////////////////////////////////////////////////////////////
// Functions
////////////////////////////////////////////////////////////////////////
Matrix4f CreateGLUPerspective( float fov_Rad, float aspectRatio, float nearClip, float farClip );
void EnableShadowTexCoord( Matrix4f value );
void DisableShadowTexCoord();
#pragma endregion

#endif // __PSM_H_INCLUDED__