#pragma region File Description
//-------------------------------------------------------------------------------------------------
// File : PixelBuffer.cpp
// Desc : Pixel Buffer Object
// Date : Dec. 01, 2008
// Version : 1.0
// Author : Pocol
//-------------------------------------------------------------------------------------------------
#pragma endregion

#pragma region Includes
//
// Includes
//
#include "PixelBuffer.h"
#pragma endregion

#pragma region PixelBuffer Class
////////////////////////////////////////////////////////////////////////
// PixelBuffer class
////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------------------------------
// Name : PixelBuffer()
// Desc : RXgN^
//--------------------------------------------------------------------------------------------------
PixelBuffer::PixelBuffer(int w, int h, HDC hdc)
: width( w ), height( h ), hPBuffer( NULL ), exists( 0 )
{
	glGenTextures( 1, &handle );
	glBindTexture( GL_TEXTURE_2D, handle );

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );

	glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexParameteri( GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_LUMINANCE );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_R_TO_TEXTURE );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, GL_LEQUAL );

	int pfAttribute[] = 
	{
		WGL_SUPPORT_OPENGL_ARB, TRUE,
		WGL_DRAW_TO_PBUFFER_ARB, TRUE,
		WGL_BIND_TO_TEXTURE_RGBA_ARB, TRUE,
		WGL_DOUBLE_BUFFER_ARB, FALSE,
		0
	};

	unsigned int nFormat = 0;
	int pixelFormat;
	wglChoosePixelFormatARB( hdc, pfAttribute, NULL, 1, &pixelFormat, &nFormat );

	if ( nFormat == 0 )
	{
		std::cout << "Error : No matching pixel format found.\n";
		return;
	}

	int pbAttribute[] =
	{
		WGL_DEPTH_TEXTURE_FORMAT_NV, WGL_TEXTURE_DEPTH_COMPONENT_NV,
		WGL_TEXTURE_FORMAT_ARB, WGL_TEXTURE_RGBA_ARB,
		WGL_TEXTURE_TARGET_ARB, WGL_TEXTURE_2D_ARB,
		0
	};

	hPBuffer = wglCreatePbufferARB( hdc, pixelFormat, width, height, pbAttribute );
	hDC = wglGetPbufferDCARB( hPBuffer );
	hRC = wglCreateContext( hDC );

	if ( !hPBuffer )
	{
		std::cout << "Error : Could not create pixel buffer.\n";
		return;
	}

	int x, y;
	wglQueryPbufferARB( hPBuffer, WGL_PBUFFER_WIDTH_ARB, &x );
	wglQueryPbufferARB( hPBuffer, WGL_PBUFFER_HEIGHT_ARB, &y );

	if ( !( x == width && y == height) )
	{
		std::cout << "Error : Size of pixel buffer is not as demanded.\n";
		return;
	}

	exists = 1;
}

//-------------------------------------------------------------------------------------------------
// Name : ~PixelBuffer()
// Desc : fXgN^
//-------------------------------------------------------------------------------------------------
PixelBuffer::~PixelBuffer()
{
	//@
	Release();

	wglDeleteContext( hRC );
	wglReleasePbufferDCARB( hPBuffer, hDC );
	wglDestroyPbufferARB( hPBuffer );
}

//--------------------------------------------------------------------------------------------------
// Name : Bind()
// Desc : oCh
//--------------------------------------------------------------------------------------------------
int PixelBuffer::Bind( GLenum drawBuffer )
{
	if ( !exists )
		return 0;
	
	if ( !wglBindTexImageARB( hPBuffer, drawBuffer ) )
		return 0;

	bound = drawBuffer;

	return 1;
}

//-------------------------------------------------------------------------------------------------
// Name : MakeCurrent()
// Desc : ReLXgJgɂ
//-------------------------------------------------------------------------------------------------
int PixelBuffer::MakeCurrent()
{
	if ( !exists ) 
		return 0;

	if ( !wglMakeCurrent( hDC, hRC ) )
		return 0;

	return 1;
}

//--------------------------------------------------------------------------------------------------
// Name : Release()
// Desc : [X
//-------------------------------------------------------------------------------------------------
int PixelBuffer::Release()
{
	if ( !exists ) 
		return 0;

	if ( !wglReleaseTexImageARB( hPBuffer, bound ) )
		return 0;

	return 1;
}
#pragma endregion